C----------------------------------------------------------------------
      SUBROUTINE ASCOUT(ARRAY,NPTS,ASCARR,GMIN,GMAX,NCHAN)
C----------------------------------------------------------------------
      IMPLICIT   NONE
C
C     PACK AND WRITE A DATA ARRAY AS A SET OF PAIRS OF CHARACTERS.
C     USES THE CHARACTERS 0-9 AND LOWER AND UPPER CASE A-Z ONLY.
C
C     INPUT ARGUMENTS:
C        ARRAY   -    REAL DATA ARRAY.
C        NPTS    -    DIMENSION OF ARRAY.
C        ASCARR  -    A CHARACTER*1 WORKSPACE ARRAY OF LENGTH 2*NPTS.
C        GMIN    -    MINIMUM VALUE OF ARRAY.
C        GMAX    -    MAXIMUM VALUE OF ARRAY.
C        NCHAN   -    FORTRAN CHANNEL FOR DATA OUTPUT.
C     ARGUMENTS UNCHANGED ON OUTPUT EXCEPT ASCARR CONTAINS CHARACTERS.
C
C     KEVIN DUNN / MIKE BLACKBURN     U.G.A.M.P.   03.01.89.
C     BASED ON CODE BY M. A. ROWE       (FRAM)        09.87.
C
      INTEGER      NPTS,NCHAN
      REAL*8       ARRAY(NPTS),GMIN,GMAX
      CHARACTER*1  ASCARR(2,NPTS),LKUP(63)
      LOGICAL      LASCII
      SAVE         LKUP
C----------------------------------------------------------------------
      INTEGER      I,N,INTEG,ICODE1,ICODE2
      REAL*8       GAMAX,RANGE,ARANG,SCALE
C----------------------------------------------------------------------
C
C     FIND SCALING FACTOR. SET TO UNITY FOR ZERO FIELD RANGE.
C
      GAMAX=MAX(ABS(GMAX),ABS(GMIN))
      RANGE=GMAX-GMIN
      ARANG=62.*62.-1.
      IF (RANGE.LE.GAMAX*1.E-10) THEN
         SCALE=0.
      ELSE
         SCALE=ARANG/RANGE
      ENDIF
C
C     SCALE DATA USING RANGE OF FIELD AND ENCODE EACH VALUE
C     USING SUBSCRIPT OF LOOK-UP TABLE CHARACTER.
C
      DO 10 N=1,NPTS
      INTEG=NINT((ARRAY(N)-GMIN)*SCALE)
      ICODE1=1+INTEG/62
      ICODE2=1+MOD(INTEG,62)
      ASCARR(1,N)=LKUP(ICODE1)
      ASCARR(2,N)=LKUP(ICODE2)
 10   CONTINUE
C
C     WRITE ENCODED DATA.
C
      WRITE(NCHAN,'(64A1)') ((ASCARR(I,N),I=1,2),N=1,NPTS)
C
      RETURN
C
C----------------------------------------------------------------------
      ENTRY ASCSET
C----------------------------------------------------------------------
C
C     DEFINE LOOK-UP TABLE OF CHARACTERS.
C     TABLE INCLUDES 0-9 AND LOWER AND UPPER CASE CHARACTERS A-Z
C     BUT NOT CONTROL CHARACTERS. ASCII OR EBCDIC CHARACTER SETS
C     ARE ALLOWED AND TEST MADE TO DETERMINE CURRENT SET.
C     CHARACTERS 1-62 IN LKUP ARE USED TO REPRESENT INTEGER VALUES
C     1-62. THE RANGE OF THE DATA FIELD IS SCALED TO 62*62 AND TWO
C     CHARACTERS USED TO REPRESENT EACH DATUM (2 DIGITS IN BASE 62).
C     63RD CHARACTER IN LKUP IS BLANK FOR POSSIBLE FUTURE MASKING.
C
C----------------------------------------------------------------------
      LASCII=ICHAR('0').NE.240
      IF (LASCII) THEN
C     IMPLEMENTATION FOR ASCII CHARACTERS.
      DO 21 I=1,10
 21   LKUP(I)=CHAR(I+47)
      DO 22 I=11,36
 22   LKUP(I)=CHAR(I+54)
      DO 23 I=37,62
 23   LKUP(I)=CHAR(I+60)
      LKUP(63)=CHAR(32)
      ELSE
C***  IMPLEMENTATION FOR EBCDIC CHARACTERS
      DO 31 I=1,10
 31   LKUP(I)=CHAR(I+239)
      DO 32 I=11,19
 32   LKUP(I)=CHAR(I+182)
      DO 33 I=20,28
 33   LKUP(I)=CHAR(I+189)
      DO 34 I=29,36
 34   LKUP(I)=CHAR(I+197)
      DO 35 I=37,45
 35   LKUP(I)=CHAR(I+92)
      DO 36 I=46,54
 36   LKUP(I)=CHAR(I+99)
      DO 37 I=55,62
 37   LKUP(I)=CHAR(I+107)
      LKUP(63)=CHAR(64)
      ENDIF
      RETURN
C
      END
