C----------------------------------------------------------------------
      SUBROUTINE DLSGCR(NCOL,F,RGG,DF,IDIM,NL)
C----------------------------------------------------------------------
      IMPLICIT   NONE
C
C     BLACK-BOX ROUTINE TO TAKE VERTICAL DERIVATIVE OF A 2D ARRAY
C     IN THE MODEL'S VERTICAL CO-ORDINATES USING THE RGG MATRIX:
C
C     DF   =   RGG   *   F
C
C     GIVES    DF   =   D(F)/DLN(SIGMA)   =   (SIGMA)D(F)/D(SIGMA).
C
C     CALCULATES DF AT NL LEVELS FOR NCOL COLUMNS FROM THE ARRAY F.
C     USES VECTOR LOOPS OVER NCOL AT EACH LEVEL.  NOTE THAT FOR
C     NCOL <= 3 INNER LOOPS UNROLL TO GIVE VECTORISATION OVER LEVELS.
C
C     USES EXPLICIT VALUES OF RGG MATRIX RATHER THAN FUNCTION SDOT
C     OVER NL ELEMENTS OF RGG AT EACH LEVEL.  DUE TO SPARSE NATURE
C     OF RGG, THIS GIVES LARGE SPEED-UP FOR NL >> 3.
C
C     READING MODEL - SIGMA.
C
C----------------------------------------------------------------------
      INTEGER  IDIM,NL,NCOL
      REAL*8   F(IDIM,NL),DF(IDIM,NL),RGG(NL,NL)
      INTEGER  NLM,I,L,NLMM
C----------------------------------------------------------------------
 6900 FORMAT(/' ***ABORT IN DLSGCR: MORE COLUMNS THAN ARRAY DIMENSION:'
     >     ,' NCOL IDIM =', 2I10)
C
      IF (NCOL.GT.IDIM) THEN
         WRITE(6,6900) NCOL,IDIM
         CALL ABORT
      ENDIF
C
      NLM=NL-1
      NLMM=NL-2
C     TOP AND BOTTOM LEVELS.
      DO 10 I=1,NCOL
      DF(I,1 )=F(I,1)*RGG(1,1)+F(I,2)*RGG(2,1)+F(I,3)*RGG(3,1)
      DF(I,NL)=F(I,NLMM)*RGG(NLMM,NL)+F(I,NLM)*RGG(NLM,NL)
     >     +F(I,NL)*RGG(NL,NL)
 10   CONTINUE
C     INTERMEDIATE LEVELS.
      DO 20 L=2,NLM
      DO 20 I=1,NCOL
      DF(I,L)=F(I,L-1)*RGG(L-1,L)+F(I,L+1)*RGG(L+1,L)
 20   CONTINUE
C
      RETURN
      END
