C----------------------------------------------------------------------
      SUBROUTINE FLUX
C----------------------------------------------------------------------
      IMPLICIT   NONE
C
C     CALCULATE ZONALLY AVERAGED FIELDS AT CURRENT ANALYSIS TIME AND
C     STORE IN COMMON ZONAV.  INCREMENT THEIR TIME AVERAGES IN COMMON
C     TAV.  USES FULL 3-DIMENSIONAL FIELDS FROM SCRATCH FILES.
C
C     AVERAGES ARE TAKEN ON ISOBARIC LEVELS, SO INTERPOLATE FROM MODEL
C     LEVELS IF NECESSARY.  IT IS ASSUMED THERE ARE NL ISOBARIC LEVELS
C     WITH P(L) = P0*SIGMA(L).  PROGRAM SHOULD HAVE ABORTED IN INIPLV
C     OR SWITCHED OFF THESE DIAGNOSTICS IF ANY OTHER PRESSURE LEVELS
C     WERE REQUESTED.
C
C     NB.  FLUX CAN BE CALLED USING DATA ON MODEL LEVELS,
C         (LINTP2=.FALSE., LINTP3=.FALSE.)
C      BUT THEN SEVERAL FIELDS ARE INVALID SINCE THEIR
C     CALCULATION ASSUMES ISOBARIC DATA: POTENTIAL TEMPERATURE, RELATIVE
C     HUMIDITY AND DERIVED DYNAMICAL FIELDS.
C     NB.  CALCULATIONS FOR SEVERAL FIELDS ASSUME ISOBARIC LEVELS ARE
C     P0*SIGMA BY USING THE FACT THAT NON-DIMENSIONAL PRESSURE IS THEN
C     JUST SIGMA.
C     NB.  THE REFERENCE PRESSURE FOR ADIABATIC PROCESSES IS ASSUMED
C     TO BE THE NON-DIMENSIONALISING P0.
C
C     NB.  FOLLOWING ZONAL ARRAYS USED AS WORKSPACE:
C          DUMNZ, DUMCN, DUMCK, DUMBP.
C
      INCLUDE   'param1.inc'
      INCLUDE   'param2.inc'
      INCLUDE   'bats.inc'
      INCLUDE   'blank.inc'
      INCLUDE   'comgen.inc'
      INCLUDE   'comioc.inc'
      INCLUDE   'comiol.inc'
      INCLUDE   'comprl.inc'
      INCLUDE   'comrog.inc'
      INCLUDE   'gridp.inc'
      INCLUDE   'legau.inc'
      INCLUDE   'phys.inc'
      INCLUDE   'tav.inc'
      INCLUDE   'zonav.inc'
      INCLUDE   'newdiagg.inc'
C
      REAL*8   DDZ(IDDZ),DDZT(IDDZ),FUBR(JGG,NL),FVBR(JGG,NL)
      REAL*8   TEMPZ(JGG,NL),RHG(IGD)
      REAL*8   ZTEM(JGG,NL),ZTEMY(JGG,NL),ZTEMP(JGG,NL)
      EQUIVALENCE (DDZ(1),UBR(1,1)),(DDZT(1),UBT(1,1))
     >           ,(RHG(1),EPVG(1))
     >           ,(FUBR(1,1),DUMCN(1,1)),(FVBR(1,1),DUMCK(1,1))
     >           ,(TEMPZ(1,1),DUMNZ(1,1))
C----------------------------------------------------------------------
      INTEGER    I,J,IH,KK,KTR,IHEM,IOF,IK,L,K,JP,JM
      REAL*8     RMGSEC,RMGSSQ,QS,SFM,SF,BP,A,FD,UMC,AG,DM
      REAL*8     SSUM,SDOT,GAV
      INTEGER    LDUM
C----------------------------------------------------------------------
C     DUMMY LEVEL FIXED TO 1 FOR TRACOLBR
      LDUM=1
C
C     REWIND SCRATCH FILES.
C
      IF (LROG.AND.JGL.EQ.1) REWIND NCROGG
      DO 10 I=1,NSG+2+NTRAC
   10 REWIND NCGP(I)
C
C     VERTICAL DERIVATIVES OF RESTORATION TEMPERATURES.
C
      CALL DLSGCR(JGG,TBRES,RGG,DUMBP,JGG,NL)
C
C     ----------------------------------------------------------------
C     MAIN LOOP OVER LATITUDE.  FIRST READ DATA.
C
      JL=1
      DO 90 IH=1,JG
      JH=IH
C
      IF (LROG.AND.JGL.EQ.1) READ(NCROGG) GSG
      READ(NCGP( 3)) DG
      READ(NCGP( 4)) OMG
      READ(NCGP( 5)) UG
      READ(NCGP( 6)) VG
      READ(NCGP( 7)) TG
      READ(NCGP( 9)) QG
      READ(NCGP(10)) HG
      READ(NCGP(11)) PLG,PMG,PJG
      READ(NCGP(13)) FUG
      READ(NCGP(14)) FVG
      DO 50 KK=1,NTRAC
      KTR=(KK-1)*IGD
 50   READ(NCGP(NSG+2+KK)) (TRAG(I),I=KTR+1,KTR+IGD)
      RMGSEC=RMG*SQRT(SECSQ(IH))
      RMGSSQ=RMG*SECSQ(IH)
C
C     INTERPOLATE TO ISOBARIC LEVELS IF REQUIRED.
C
      IF (LINTP3.AND.(.NOT.LINTP2)) THEN
         CALL PSGCR
         CALL SRFCR
         CALL INTPR
      ENDIF
C
      DO 80 IHEM=1,NHEM
      IOF=(IHEM-1)*MGPP
      IK=(3-2*IHEM)*IH+(JGG+1)*(IHEM-1)
C
C     CALCULATE POTENTIAL TEMPERATURE AND RELATIVE HUMIDITY.
C     USE FORMULA FROM SGCM, PDN=BGCM5,ID=UPLIB,ED=5, FOR QS(T,P).
C
      DO 20 L=1,NL
      K=(L-1)*IGC+IOF
      DO 20 I=1,MG
      K=K+1
      THG(K)=TG(K)*SIGMAK(L)
      QS=ESCONA*EXP(-ESCONB/TG(K))/SIGMA(L)
      RHG(K)=QG(K)/QS
 20   CONTINUE
C
C     FORM ZONAL AVERAGES OF BASIC FIELDS AND VARIANCES.
C     ARRAYS IN ZONAV HAVE POINT NEAREST NORTH POLE STORED FIRST
C     AND, FOR NHEM=2, POINT NEAREST SOUTH POLE STORED LAST.
C
      SFM=0.
      DO 30 L=1,NL
      I=(L-1)*IGC+IOF+1
      UBR(IK,L) =SSUM(MG,UG(I) ,1)*RMGSEC
      VBR(IK,L) =SSUM(MG,VG(I) ,1)*RMGSEC
      WBR(IK,L) =SSUM(MG,OMG(I),1)*RMG
      TBR(IK,L) =SSUM(MG,TG(I) ,1)*RMG
      QBR(IK,L) =SSUM(MG,QG(I) ,1)*RMG
      HBR(IK,L) =SSUM(MG,HG(I) ,1)*RMG
      RHBR(IK,L)=SSUM(MG,RHG(I),1)*RMG
      THBR(IK,L)=SSUM(MG,THG(I),1)*RMG
      FUBR(IK,L)=SSUM(MG,FUG(I),1)*RMGSEC
      FVBR(IK,L)=SSUM(MG,FVG(I),1)*RMGSEC
      UVBR(IK,L)=SDOT(MG,UG(I),1,VG(I) ,1)*RMGSSQ
      UWBR(IK,L)=SDOT(MG,UG(I),1,OMG(I),1)*RMGSEC
      VTBR(IK,L)=SDOT(MG,VG(I),1,TG(I) ,1)*RMGSEC
      WTBR(IK,L)=SDOT(MG,TG(I),1,OMG(I),1)*RMG
      VQBR(IK,L)=SDOT(MG,VG(I),1,QG(I) ,1)*RMGSEC
      WQBR(IK,L)=SDOT(MG,QG(I),1,OMG(I),1)*RMG
      UUBR(IK,L)=SDOT(MG,UG(I),1,UG(I) ,1)*RMGSSQ
      VVBR(IK,L)=SDOT(MG,VG(I),1,VG(I) ,1)*RMGSSQ
      TTBR(IK,L)=SDOT(MG,TG(I),1,TG(I) ,1)*RMG
      HTBR(IK,L)=SDOT(MG,HG(I),1,TG(I) ,1)*RMG
      VWBR(IK,L)=SDOT(MG,VG(I),1,OMG(I),1)*RMGSEC
      FUER(IK,L)=(SDOT(MG,FUG(I),1,UG(I),1)
     >           +SDOT(MG,FVG(I),1,VG(I),1))*RMGSSQ
      DO 60 KK=1,NTRAC
      KTR=(KK-1)*IGD
      TRABR(IK,L,KK)=SSUM(MG,TRAG(I+KTR),1)*RMG
      VTRABR(IK,L,KK)=SDOT(MG,VG(I),1,TRAG(I+KTR),1)*RMGSEC
      WTRABR(IK,L,KK)=SDOT(MG,TRAG(I+KTR),1,OMG(I),1)*RMG
      TRACOLBR(IK,LDUM,KK)=SSUM(MG,TRACOL(1+IOF,JH,KK),1)*RMG
 60   CONTINUE
      SF=SFM+DSIGMA(L)*VBR(IK,L)*CS(IH)
      SFMCR(IK,L)=.5*(SF+SFM)
      SFM=SF
 30   CONTINUE
C
      BP=SSUM(MG,PLG(IOF+1),1)*RMG-1.
      DO 40 L=1,NL
      TMTRES(IK,L)=TBR(IK,L)-(TBRES(IK,L)-BP*DUMBP(IK,L))
 40   CONTINUE
C
 80   CONTINUE
C
 90   JL=JL+JINC
C
C     ----------------------------------------------------------------
C     END OF MAIN LATITUDE LOOP.
C
C     CALCULATE LAYER MEAN AND GLOBAL MEAN VALUES OF OMEGA,
C     TEMPERATURE AND DIABATIC HEATING.
C
      WMNN=GAV(WBR,GWT,JGG,DSIGMA,NL,WMN)
      TMNN=GAV(TBR,GWT,JGG,DSIGMA,NL,TMN)
      HMNN=GAV(HBR,GWT,JGG,DSIGMA,NL,HMN)
C
C     SEPARATE ZONAL AND EDDY VARIANCES, CALCULATE OTHER DERIVED FIELDS.
C     NB.  TO PREVENT CONTAMINATION OF ENERGY CONVERSIONS, REMOVE
C     ISOBARIC MEANS OF OMEGA, TEMPERATURE AND HEATING FROM MEAN HEAT
C     FLUX AND HEATING ARRAYS (VBTBR, WBTBR, HBTBR).
C     NB.  LAYER MEAN OMEGA IS THEORETICALLY ZERO ON ALL GLOBAL ISOBARIC
C     SURFACES ABOVE OROGRAPHY BUT NUMERICAL ESTIMATES ARE GENERALLY NON
C     ZERO.  IT DOES NOT ENTER THE GLOBAL ENERGETICS SO, IN ORDER NOT TO
C     CONTAMINATE THE RESULTS, IS REMOVED AT ALL LEVELS IRRESPECTIVE OF
C     WHETHER ABOVE OR BELOW OROGRAPHY, MODEL-LEVEL OR ISOBARIC. IT IS
C     NOT REMOVED FROM THE OMEGA ARRAY ITSELF.  IF THIS IS DONE, IT MUST
C     BE AFTER SEPARATING THE ZONAL AND EDDY VARIANCES, OR THE VERTICAL
C     EDDY FLUXES WILL BE CONTAMINATED.
C
      DO 100 L=1,NL
      DO 100 J=1,JGG
      UVBR(J,L)=UVBR(J,L)-UBR(J,L)*VBR(J,L)
      UWBR(J,L)=UWBR(J,L)-UBR(J,L)*WBR(J,L)
      VTBR(J,L)=VTBR(J,L)-VBR(J,L)*TBR(J,L)
      WTBR(J,L)=WTBR(J,L)-WBR(J,L)*TBR(J,L)
      VQBR(J,L)=VQBR(J,L)-VBR(J,L)*QBR(J,L)
      WQBR(J,L)=WQBR(J,L)-WBR(J,L)*QBR(J,L)
      UUBR(J,L)=UUBR(J,L)-UBR(J,L)*UBR(J,L)
      VVBR(J,L)=VVBR(J,L)-VBR(J,L)*VBR(J,L)
      TTBR(J,L)=TTBR(J,L)-TBR(J,L)*TBR(J,L)
      HTBR(J,L)=HTBR(J,L)-HBR(J,L)*TBR(J,L)
      VWBR(J,L)=VWBR(J,L)-VBR(J,L)*WBR(J,L)
      AKZ (J,L)=.5*(UBR(J,L)*UBR(J,L)+VBR(J,L)*VBR(J,L))
      AKE (J,L)=.5*(UUBR(J,L)+VVBR(J,L))
      FUZR(J,L)=FUBR(J,L)*UBR(J,L)+FVBR(J,L)*VBR(J,L)
      FUER(J,L)=FUER(J,L)-FUZR(J,L)
      VBTBR(J,L)=VBR(J,L)*(TBR(J,L)-TMN(L))
      WBTBR(J,L)=(WBR(J,L)-WMN(L))*(TBR(J,L)-TMN(L))
      HBTBR(J,L)=(HBR(J,L)-HMN(L))*(TBR(J,L)-TMN(L))
      DO 70 KK=1,NTRAC
      KTR=(KK-1)*IGD
      VTRABR(J,L,KK)=VTRABR(J,L,KK)-VBR(J,L)*TRABR(J,L,KK)
      WTRABR(J,L,KK)=WTRABR(J,L,KK)-WBR(J,L)*TRABR(J,L,KK)
 70   CONTINUE
 100  CONTINUE
C
C     DERIVE REMAINING ZONALLY AVERAGED QUANTITIES.
C
C     CALCULATE TERMS INVOLVING FIRST MERIDIONAL DERIVATIVE.
C
      DO 110 L=1,NL
      DO 110 J=2,JGGM
      JM=J-1
      JP=J+1
      DM=1./(SI(JP)-SI(JM))
      ABSV(J,L)=SI(J)+SI(J)-(UBR(JP,L)*CS(JP)-UBR(JM,L)*CS(JM))*DM
      DUBDMU(J,L)=(UBR(JP,L)/CS(JP)-UBR(JM,L)/CS(JM))*DM
      DVBDMU(J,L)=(VBR(JP,L)/CS(JP)-VBR(JM,L)/CS(JM))*DM
      DTBDMU(J,L)=(TBR(JP,L)-TBR(JM,L))*DM
 110  CONTINUE
C
C     EXTRAPOLATE TO NORTHERN- AND SOUTHERNMOST GAUSSIAN LATITUDES.
C
      DO 120 L=1,NL
      ABSV(  1,L)  =EXN1*ABSV(   2,L)  -EXN2*ABSV(    3,L)
      ABSV(JGG,L)  =EXS1*ABSV(JGGM,L)  -EXS2*ABSV(JGGM2,L)
      DUBDMU(  1,L)=EXN1*DUBDMU(   2,L)-EXN2*DUBDMU(    3,L)
      DUBDMU(JGG,L)=EXS1*DUBDMU(JGGM,L)-EXS2*DUBDMU(JGGM2,L)
      DVBDMU(  1,L)=EXN1*DVBDMU(   2,L)-EXN2*DVBDMU(    3,L)
      DVBDMU(JGG,L)=EXS1*DVBDMU(JGGM,L)-EXS2*DVBDMU(JGGM2,L)
      DTBDMU(  1,L)=EXN1*DTBDMU(   2,L)-EXN2*DTBDMU(    3,L)
      DTBDMU(JGG,L)=EXS1*DTBDMU(JGGM,L)-EXS2*DTBDMU(JGGM2,L)
 120  CONTINUE
C
C     CALCULATE TERMS INVOLVING FIRST VERTICAL DERIVATIVE.
C
      CALL DLSGCR(JGG,UBR,RGG,UBRP,JGG,NL)
      CALL DLSGCR(JGG,VBR,RGG,VBRP,JGG,NL)
      CALL DLSGCR(JGG,TBR,RGG,STBP,JGG,NL)
      AG=1.
      DO 130 L=1,NL
      DO 130 J=1,JGG
      UBRP(J,L)=UBRP(J,L)/SIGMA(L)
      VBRP(J,L)=VBRP(J,L)/SIGMA(L)
      THBP(J,L)=(STBP(J,L)-AKAP*TBR(J,L))/SIGMA(L)
      ENSQ(J,L)=-THBP(J,L)*SIGMA(L)/(TBR(J,L)*TBR(J,L))
      TEMPZ(J,L)=UBRP(J,L)*SIGMA(L)/THBP(J,L)
      EPFH(J,L)=CSSQ(J)*(-UVBR(J,L)+AG*UBRP(J,L)*VTBR(J,L)/THBP(J,L))
      EPFV(J,L)=CSSQ(J)*(AG*UWBR(J,L)-ABSV(J,L)*VTBR(J,L)/THBP(J,L))
      ZTEM(J,L)=VTBR(J,L)/THBP(J,L)
      TEMSFBR(J,L)=SFMCR(J,L)-CS(J)*ZTEM(J,L)
 130  CONTINUE
C
C     CALCULATE TERMS INVOLVING SECOND MERIDIONAL DERIVATIVE.
C
      DO 140 L=1,NL
      DO 140 J=2,JGGM
      JM=J-1
      JP=J+1
      DM=1.0/(SI(JP)-SI(JM))
      QY(J,L)=(ABSV(JP,L)-ABSV(JM,L))*DM
      EPDH(J,L)=CS(J)*(EPFH(JP,L)-EPFH(JM,L))*DM
      ZTEMY(J,L)=(ZTEM(JP,L)*CS(JP)-ZTEM(JM,L)*CS(JM))*DM
 140  CONTINUE
C
C     EXTRAPOLATE TO NORTHERN- AND SOUTHERNMOST GAUSSIAN LATITUDES.
C
      DO 150 L=1,NL
      QY(  1,L)  =EXN1*QY(   2,L)  -EXN2*QY(    3,L)
      QY(JGG,L)  =EXS1*QY(JGGM,L)  -EXS2*QY(JGGM2,L)
C     MODIF M. BOURQUI 24.JUNE 2002
C      EPDH(1,L)=(EXN1*EPDH(2,L)/CSSQ(2)-EXN2*EPDH(3,L)/CSSQ(3))*CSSQ(1)
C      EPDH(JGG,L)=(EXS1*EPDH(JGGM,L)/CSSQ(JGGM)-
C     &                        EXS2*EPDH(JGGM2,L)/CSSQ(JGGM2))*CSSQ(JGG)
C     NEW MODIF M. BOURQUI 27. JUNE 2002
      EPDH(1,L)=(SI(1)-1.)*EPDH(2,L)*CSSQ(1)/((SI(2)-1.)*CSSQ(2))
      EPDH(JGG,L)=(SI(JGG)-1.)*EPDH(JGGM,L)*CSSQ(JGG)/
     &                  ((SI(JGGM)-1.)*CSSQ(JGGM))
      ZTEMY(  1,L)=EXN1*ZTEMY(   2,L)-EXN2*ZTEMY(    3,L)
      ZTEMY(JGG,L)=EXS1*ZTEMY(JGGM,L)-EXS2*ZTEMY(JGGM2,L)
 150  CONTINUE
C
C     CALCULATE TERMS INVOLVING SECOND VERTICAL DERIVATIVES.
C     FORM TOTAL E-P FLUX DIVERGENCE.
C
      CALL DLSGCR(JGG,TEMPZ,RGG,DUMBP,JGG,NL)
      CALL DLSGCR(JGG,EPFV,RGG,EPDV,JGG,NL)
      CALL DLSGCR(JGG,ZTEM,RGG,ZTEMP,JGG,NL)
      DO 160 L=1,NL
      DO 165 J=1,JGG
      QY(J,L)=QY(J,L)*CS(J)+DUMBP(J,L)*4.*SISQ(J)/SIGMA(L)
      EPDV(J,L)=-EPDV(J,L)/SIGMA(L)
      EPDT(J,L)=EPDH(J,L)+EPDV(J,L)
      TEMVBR(J,L)=VBR(J,L)-ZTEMP(J,L)/SIGMA(L)
      TEMWBR(J,L)=WBR(J,L)+ZTEMY(J,L)
 165  CONTINUE
C     MODIF M. BOURQUI 27. JUNE 2002
      EPDT(1,L)=(EXN1*EPDT(2,L)/CSSQ(2)-EXN2*EPDT(3,L)/CSSQ(3))*CSSQ(1)
      EPDT(JGG,L)=(EXS1*EPDT(JGGM,L)/CSSQ(JGGM)-
     &                        EXS2*EPDT(JGGM2,L)/CSSQ(JGGM2))*CSSQ(JGG)
 160  CONTINUE
C     TEST
C      DO L=1,NL
C      DO J=1,JGG
C      WRITE(26,*)J,JG,L,EPDH(J,L),EPDV(J,L),EPDT(J,L)
C      ENDDO
C      ENDDO
C
C     CALCULATE MAXIMUM PERMITTED WAVENUMBER FOR WAVES WITH SPECIFIED
C     EASTWARD PHASE SPEED.
C
      DO 170 J=1,JGG
      DO 170 L=1,NL
      UMC=UBR(J,L)/CS(J)-CPHASE
      IF (UMC.EQ.0.) THEN
         A=1.E4
      ELSE
         A=QY(J,L)*CS(J)/UMC
         IF (A.LT.0.) A=1.E4
      ENDIF
      TPWN(J,L)=SQRT(A)
 170  CONTINUE
C
C     SET UP FACTOR FOR TIME-AVERAGING.
C     INCREMENT TIME AVERAGES OF ZONALLY AVERAGED ARRAYS.
C
      IF (LTAV) THEN
         FD=FTAV
         IF (KOUNT.EQ.KSTART.OR.KOUNT.EQ.KEND) FD=.5*FD
         DO 180 I=1,IDDZ
 180     DDZT(I)=DDZT(I)+FD*DDZ(I)
      ENDIF
C
C     REWIND SCRATCH FILES.
C
      IF (LROG.AND.JGL.EQ.1) REWIND NCROGG
      DO 190 I=1,NSG+2+NTRAC
 190  REWIND NCGP(I)
C
      RETURN
      END
