C----------------------------------------------------------------------
      SUBROUTINE GDIAG
C----------------------------------------------------------------------
      IMPLICIT   NONE
C
C     PERFORM GLOBAL ENERGETICS ANALYSIS USING ENTROPIC POTENTIAL
C     ENERGY (EPE) AND KINETIC ENERGY (KE).  THE SCHEME IS DEFINED IN
C     BLACKBURN (1983, PHD THESIS, UNIVERSITY OF READING) AND IS A
C     MODIFIED VERSION OF THAT DEFINED BY PEARCE (1978, QJRMS, 104,
C     PP737-755).
C
C                  L------L                L------L
C          --GZ----L  NZ  L------CZZ-------L  KZ  L----DZ--
C                  L------L                L------L
C                     L    \              /   L
C                     L    CNSZ        CSZ    L
C                     L       \        /      L
C                    CNZE      L------L       L
C          --GS-------L--------L  NS  L     CKZE
C                     L        L------L       L
C                     L       /        \      L
C                     L    CNSE        CSE    L
C                     L    /              \   L
C                  L------L                L------L
C          --GE----L  NE  L------CEE-------L  KE  L----DE--
C                  L------L                L------L
C
C     EXTRA TERMS CALCULATED ARE:
C          TE = NE  + KE     TOTAL EDDY ENERGY,
C          CZ = CZZ + CSZ    TOTAL CONVERSION FROM EPE TO KZ,
C          CE = CEE + CSE    TOTAL CONVERSION FROM EPE TO KE.
C     ENERGIES ARE GIVEN IN JOULES/M2 AND CONVERSIONS IN WATTS/M2.
C     FOR VALUES IN J/KG AND W/KG, DIVIDE BY THE UNIT AREA MASS, P0/GA.
C     ALL GLOBAL INTEGRALS ARE OVER MASS AND GIVE EQUAL WEIGHTING TO
C     ALL COLUMNS BY INTEGRATING FROM P=0 TO P=P0.
C
C     THIS ROUTINE IS CALLED AT EACH ANALYSIS TIME TO COMPUTE THE
C     INSTANTANEOUS ENERGIES AND CONVERSIONS AND TO INCREMENT THE
C     TIME AVERAGES.  THE TIME SERIES IS BUILT UP ON CHANNEL NCGEN AND
C     IS READ AT THE END OF THE RUN WHEN GDIAG IS CALLED WITH KOUNT<0.
C     THE TIME SERIES AND AVERAGES ARE THEN PRINTED AND OPTIONALLY
C     WRITTEN TO CHANNEL NCPLOT(1) FOR PLOTTING IN A SUBSEQUENT PROGRAM.
C
      INCLUDE   'param1.inc'
      INCLUDE   'param2.inc'
      INCLUDE   'bats.inc'
      INCLUDE   'blank.inc'
      INCLUDE   'comgen.inc'
      INCLUDE   'comioc.inc'
      INCLUDE   'comiol.inc'
      INCLUDE   'legau.inc'
      INCLUDE   'outcon.inc'
      INCLUDE   'series.inc'
      INCLUDE   'zonav.inc'
C
      REAL*8       STAB(NL),GAMDS(NL),DSRSI(NL),SDSRSI(NL),TEMP(NL)
      REAL*8       TMSQ(NL),HMTM(NL),WTMN(NL)
      REAL*8       TEQV(NTERM5),SEQV(NTERM5)
      EQUIVALENCE (TEQV(1),TNS),(SEQV(1),SNS)
      CHARACTER    IYLAB*30,IYLABA*30,IYLABB*30,IXLAB*22,ICAN(6)*3
      CHARACTER    ICANA(15)*3,LTEMP*8
      SAVE IYLAB,IYLABA,IYLABB,IXLAB,ICAN,LTEMP
C
      DATA IYLAB /'              ENERGIES*.'/
      DATA IYLABA/'          ENERGY CONVERSIONS*.'/
      DATA IYLABB/'          KE IN VARIOUS M*.'/
      DATA IXLAB /'    TIME IN DAYS*.'/
      DATA LTEMP /'      *.'/
      DATA ICAN  /' NS',' NZ',' NE',' KZ',' KE','   '/
     >    ,ICANA /' CZ',' CE','NZE','KZE','NSZ','NSE'
     >           ,' GS',' GZ',' GE',' DZ',' DE'
     >           ,'IGZ','IGE','IDZ','IDE'/
C----------------------------------------------------------------------
      INTEGER   I,J,L,I1,NI,NPLOT,I2,K,IXI,NEND,NBEG,IANC,IYT,IXT
      INTEGER   JOUNT,N
      REAL*8    XL,XMN,YL,XU,TEE,TDAY,FD,TCE,TCZ,CFAC,EFAC,STMNN,GAML
      REAL*8    GAV,SDOT
C----------------------------------------------------------------------
C
 6000 FORMAT(' GLOBAL ENERGETICS AT DAY ',F10.2)
 6010 FORMAT(' TIME AVERAGE GLOBAL DIAGNOSTICS')
 6020 FORMAT(' GLOBAL MEANS: TEMP  OMEGA  HEATING  STABILITY =',4F12.5)
 6030 FORMAT(' ***GDIAG : TIME SERIES OF GLOBAL ENERGETICS ON PLOT FILE'
     >      ,' STOPS AT DAY',F10.2)
 6040 FORMAT(' ENERGETICS TIME SERIES RUNS FROM DAY ',F10.2
     >      ,' TO DAY ',F10.2)
 6100 FORMAT(1P,'     NS=',E11.4,'     NZ=',E11.4
     >         ,'     NE=',E11.4,'     KZ=',E11.4
     >         ,'     KE=',E11.4,'     TE=',E11.4,'    J/M2')
 6110 FORMAT(1P,'   CNZE=',E11.4,'    CZZ=',E11.4,'    CSZ=',E11.4
     >         ,'   CNSZ=',E11.4,'     CZ=',E11.4,'    W/M2'
     >         /'   CKZE=',E11.4,'    CEE=',E11.4,'    CSE=',E11.4
     >         ,'   CNSE=',E11.4,'     CE=',E11.4,'    W/M2')
 6120 FORMAT(1P,'     GS=',E11.4,'     GZ=',E11.4,'     GE=',E11.4
     >         ,'     DZ=',E11.4,'     DE=',E11.4,'    W/M2')
 6130 FORMAT(1P,'        ',  11X,'    IGZ=',E11.4,'    IGE=',E11.4
     >         ,'    IDZ=',E11.4,'    IDE=',E11.4,'    W/M2')
 6140 FORMAT(1P,'    RNS=',E11.4,'    RNZ=',E11.4,'    RKZ=',E11.4
     >         ,'    DRZ=',E11.4,'    DRE=',E11.4,'    W/M2')
C
      IF (KOUNT.LT.0) GOTO 100
C
C     ----------------------------------------------------------------
C
C     CALCULATE GLOBAL AVERAGES FOR ONE ANALYSIS TIME
C     AND INCREMENT TIME AVERAGES.
C
      IF (KOUNT.EQ.KSTART) THEN
         NPTS5=0
         REWIND NCGEN
         IF (LTAV) THEN
            DO 10 N=1,NTERM5
   10       SEQV(N)=0.
         ENDIF
      ENDIF
C
C     ZONAL AND ISOBARIC MEANS REQUIRED FOR GLOBAL ENERGETICS.
C
      CALL DLSGCR(1,TMN,RGG,STAB,1,NL)
      GAML=1./(AKAP*TMNN)
      DO 20 L=1,NL
      STAB(L)=GAML*STAB(L)
      GAMDS(L)=GAML*DSIGMA(L)
      DSRSI(L)=DSIGMA(L)/SIGMA(L)
      SDSRSI(L)=STAB(L)*DSRSI(L)
      TMSQ(L)=(TMN(L)-TMNN)*(TMN(L)-TMNN)
      HMTM(L)=(HMN(L)-HMNN)*(TMN(L)-TMNN)
   20 CONTINUE
      STMNN=SDOT(NL,STAB(1),1,DSIGMA(1),1)
      DO 30 L=1,NL
      WTMN(L)=SDOT(JGG,WTBR(1,L),1,GWT(1),1)
      DO 30 J=1,JGG
      DUMNZ(J,L)=(TBR(J,L)-TMN(L))*(TBR(J,L)-TMN(L))
      DUMCN(J,L)=VTBR(J,L)*DTBDMU(J,L)*GAML*CS(J)
     >          +(WTBR(J,L)-WTMN(L))*(STBP(J,L)*GAML-STAB(L))/SIGMA(L)
      DUMCK(J,L)=CSSQ(J)*(UVBR(J,L)*DUBDMU(J,L)+VVBR(J,L)*DVBDMU(J,L))
     >          +UWBR(J,L)*UBRP(J,L)+VWBR(J,L)*VBRP(J,L)
     >          -VBR(J,L)*AKE(J,L)*2.*SI(J)/CS(J)
   30 CONTINUE
C
C     CALCULATE GLOBALLY AVERAGED ENERGETICS VALUES.
C
      EFAC=CG*P0/GA
      CFAC=EFAC*WW
      TNZ= GAV(DUMNZ,GWT,JGG,GAMDS ,NL,TEMP)*EFAC*.5
      TNE= GAV(TTBR ,GWT,JGG,GAMDS ,NL,TEMP)*EFAC*.5
      TKZ= GAV(AKZ  ,GWT,JGG,DSIGMA,NL,TEMP)*EFAC
      TKE= GAV(AKE  ,GWT,JGG,DSIGMA,NL,TEMP)*EFAC
      TGZ= GAV(HBTBR,GWT,JGG,GAMDS ,NL,TEMP)*CFAC
      TGE= GAV(HTBR ,GWT,JGG,GAMDS ,NL,TEMP)*CFAC
      TDZ=-GAV(FUZR ,GWT,JGG,DSIGMA,NL,TEMP)*CFAC
      TDE=-GAV(FUER ,GWT,JGG,DSIGMA,NL,TEMP)*CFAC
      TCZ=-GAV(WBTBR,GWT,JGG,DSRSI ,NL,TEMP)*CFAC
      TCE=-GAV(WTBR ,GWT,JGG,DSRSI ,NL,TEMP)*CFAC
      TCSZ=TCZ*STMNN
      TCSE=TCE*STMNN
      TCZZ=TCZ-TCSZ
      TCEE=TCE-TCSE
      TCNSZ=-GAV(WBTBR,GWT,JGG,SDSRSI,NL,TEMP)*CFAC - TCSZ
      TCNSE=-GAV(WTBR ,GWT,JGG,SDSRSI,NL,TEMP)*CFAC - TCSE
      TCNZE=-GAV(DUMCN,GWT,JGG,DSIGMA,NL,TEMP)*CFAC
      TCKZE=-GAV(DUMCK,GWT,JGG,DSIGMA,NL,TEMP)*CFAC
      TNS = SDOT(NL,TMSQ(1)  ,1,GAMDS(1),1)*EFAC*.5
      TGS = SDOT(NL,HMTM(1)  ,1,GAMDS(1),1)*CFAC
      FIGZ=-SDOT(NL,GAMDS(1) ,1,TIGZ(1) ,1)*AKK
      FIGE=-SDOT(NL,GAMDS(1) ,1,TIGE(1) ,1)*AKK
      FIDZ= SDOT(NL,DSIGMA(1),1,TIDZ(1) ,1)*AKK
      FIDE= SDOT(NL,DSIGMA(1),1,TIDE(1) ,1)*AKK
      FRNS= SDOT(NL,DSIGMA(1),1,TRNS(1) ,1)*DAMP
      FRNZ=-SDOT(NL,GAMDS(1) ,1,TRNZ(1) ,1)*DAMP
      FRKZ= SDOT(NL,DSIGMA(1),1,TRKZ(1) ,1)*DAMP
      FDRZ= SDOT(NL,DSIGMA(1),1,TDRZ(1) ,1)
      FDRE= SDOT(NL,DSIGMA(1),1,TDRE(1) ,1)
C
C     ACCUMULATE TIME AVERAGES.
C
      IF (LTAV) THEN
         FD=FTAV
         IF (KOUNT.EQ.KSTART.OR.KOUNT.EQ.KEND) FD=.5*FD
         DO 40 N=1,NTERM5
   40    SEQV(N)=SEQV(N)+TEQV(N)*FD
      ENDIF
C
C     WRITE INSTANTANEOUS VALUES AND ALSO TIME AVERAGES AT KEND.
C
      WRITE(NCGEN) KOUNT,DAY
      WRITE(NCGEN) (TEQV(N),N=1,NTERM5),TMNN,WMNN,HMNN,STMNN
      IF (KOUNT.EQ.KEND.AND.LTAV) THEN
         JOUNT=-999
         TDAY=-999.
         WRITE(NCGEN) JOUNT,TDAY
         WRITE(NCGEN) (SEQV(N),N=1,NTERM5),TDAY,TDAY,TDAY,TDAY
      ENDIF
C
C     COPY ENERGETICS VALUES INTO TIME SERIES ARRAYS FOR PLOTTING.
C     THESE ARE SORTED AND COPIED TO PLOT FILE AT END OF RUN.
C
      IF (LENCOP.AND.NPTS5.LT.NPMAX5) THEN
         NPTS5=NPTS5+1
         IF (NPTS5.EQ.NPMAX5) WRITE(6,6030) DAY
         ADAY(NPTS5)=DAY
         AEN(NPTS5,1)=TNS
         AEN(NPTS5,2)=TNZ
         AEN(NPTS5,3)=TNE
         AEN(NPTS5,4)=TKZ
         AEN(NPTS5,5)=TKE
         ACONV(NPTS5, 1)= TCZ
         ACONV(NPTS5, 2)= TCE
         ACONV(NPTS5, 3)= TCNZE
         ACONV(NPTS5, 4)= TCKZE
         ACONV(NPTS5, 5)= TCNSZ
         ACONV(NPTS5, 6)= TCNSE
         ACONV(NPTS5, 7)= TGS
         ACONV(NPTS5, 8)= TGZ
         ACONV(NPTS5, 9)= TGE
         ACONV(NPTS5,10)=-TDZ
         ACONV(NPTS5,11)=-TDE
         ACONV(NPTS5,12)= FIGZ
         ACONV(NPTS5,13)= FIGE
         ACONV(NPTS5,14)=-FIDZ
         ACONV(NPTS5,15)=-FIDE
      ENDIF
C
      RETURN
C
C     ----------------------------------------------------------------
C
C     END OF RUN. READ TIME SERIES OF GLOBAL TERMS AND PRINT.
C
  100 CONTINUE
      REWIND NCGEN
      IF (NPC5.NE.6) WRITE(NPC5,'(/////)')
      WRITE(6,'(/////)')
C
  110 CONTINUE
      READ(NCGEN) JOUNT,TDAY
      READ(NCGEN) (TEQV(N),N=1,NTERM5),TMNN,WMNN,HMNN,STMNN
      IF (JOUNT.GE.0) THEN
         IF (NPC5.NE.6) WRITE(NPC5,6000) TDAY
         WRITE(6,6000) TDAY
      ELSE
         IF (NPC5.NE.6) WRITE(NPC5,6010)
         WRITE(6,6010)
      ENDIF
      IF (JOUNT.GE.0) THEN
         WMNN=WMNN*CMBHR
         TMNN=TMNN*CT
         HMNN=HMNN*CTT
         IF (NPC5.NE.6) WRITE(NPC5,6020) TMNN,WMNN,HMNN,STMNN
         WRITE(6,6020) TMNN,WMNN,HMNN,STMNN
      ENDIF
      TEE=TKE+TNE
      TCZ=TCZZ+TCSZ
      TCE=TCEE+TCSE
      IF (NPC5.NE.6) THEN
         WRITE(NPC5,6100) TNS,TNZ,TNE,TKZ,TKE,TEE
         WRITE(NPC5,6110) TCNZE,TCZZ,TCSZ,TCNSZ,TCZ
     >                   ,TCKZE,TCEE,TCSE,TCNSE,TCE
         WRITE(NPC5,6120) TGS,TGZ,TGE,TDZ,TDE
         WRITE(NPC5,6130) FIGZ,FIGE,FIDZ,FIDE
         WRITE(NPC5,6140) FRNS,FRNZ,FRKZ,FDRZ,FDRE
      ENDIF
      WRITE(6,6100) TNS,TNZ,TNE,TKZ,TKE,TEE
      WRITE(6,6110) TCNZE,TCZZ,TCSZ,TCNSZ,TCZ
     >             ,TCKZE,TCEE,TCSE,TCNSE,TCE
      WRITE(6,6120) TGS,TGZ,TGE,TDZ,TDE
      WRITE(6,6130) FIGZ,FIGE,FIDZ,FIDE
      WRITE(6,6140) FRNS,FRNZ,FRKZ,FDRZ,FDRE
C
      IF (JOUNT.GE.0.AND.JOUNT.LT.KEND) GOTO 110
      IF (JOUNT.EQ.KEND.AND.LTAV) GOTO 110
C
C     SORT AND WRITE TIME SERIES OF ENERGIES AND CONVERSIONS
C     TO PLOT FILE FOR USE IN SUBSEQUENT JOB IF REQUIRED.
C
      IF (.NOT.LENCOP) RETURN
      IXT=3
      IYT=3
      IANC=-1
      XU=XUEN
      YL=335.
      WRITE(IYLAB (1:8),'(F8.3)') RNTAPE
      WRITE(IYLABA(1:8),'(F8.3)') RNTAPE
      WRITE(IYLABB(1:8),'(F8.3)') RNTAPE
      DO 200 L=1,5
  200 ENMX(L)=ENMN(L)+ENRNGE
C
      NBEG=1
      NEND=NBEG+ISPLIT
  210 IF (NEND.GT.NPTS5) NEND=NPTS5
      IF (NPC5.NE.6) WRITE(NPC5,6040) ADAY(NBEG),ADAY(NEND)
      WRITE(6,6040) ADAY(NBEG),ADAY(NEND)
      XMN=ADAY(NBEG)
      XL=ADAY(NEND)-XMN
      IXI=NINT(XL)
      XL=XL/XU
      N=NEND-NBEG+1
C
      LNPIC=.TRUE.
      DO 220 I=1,5
      LTEMP(4:6)=ICAN(I)
      WRITE(NCPLOT(1)) N,IXT,IYT,LTEMP,IANC,IXLAB,IYLAB,XL,XU,XMN
     >         ,IXI,YL,ENMX(I),ENMN(I),IYEN,LNPIC
      WRITE(NCPLOT(1)) (ADAY(K),K=NBEG,NEND),(AEN(K,I),K=NBEG,NEND)
  220 LNPIC=.FALSE.
C
      I2=0
      DO 240 NPLOT=1,3
      IF (NPLOT.EQ.1) NI=6
      IF (NPLOT.EQ.2) NI=5
      IF (NPLOT.EQ.3) NI=4
      I1=I2+1
      I2=I1+NI-1
      LNPIC=.TRUE.
      DO 230 I=I1,I2
      LTEMP(4:6)=ICANA(I)
      WRITE(NCPLOT(1)) N,IXT,IYT,LTEMP,IANC,IXLAB,IYLABA,XL,XU,XMN
     >         ,IXI,YL,ECONMX,ECONMN,IYCON,LNPIC
      WRITE(NCPLOT(1)) (ADAY(K),K=NBEG,NEND),(ACONV(K,I),K=NBEG,NEND)
  230 LNPIC=.FALSE.
  240 CONTINUE
C
      NBEG=NEND
      NEND=NBEG+ISPLIT
      IF (NBEG.LT.NPTS5) GOTO 210
C
      RETURN
      END
