C----------------------------------------------------------------------
      SUBROUTINE GPREP(G,JXY,AFAC,CINC,CL,NCNT,GNORM,GINC,ICNT
     >                ,GMIN,GMAX,IFAIL)
C----------------------------------------------------------------------
      IMPLICIT   NONE
C----------------------------------------------------------------------
      INTEGER    JXY,NCNT,ICNT,IFAIL
      REAL*8     G(JXY),CL(NCNT),AFAC,CINC,GNORM,GINC,GMIN,GMAX
C
C     PREPARE AN ARRAY FOR PRINTING AND PLOTTING.
C     USE AFAC TO SET UP GNORM MULTIPLYING FACTOR FOR PRINTING.
C     USE CINC TO SET UP CONTOUR LEVELS CL(NCNT) FOR PLOTTING.
C     NB.  FIELD IS NOT SCALED OR OVERWRITTEN. CONTOUR LEVELS ARE
C     CONVERTED TO SAME DIMENSIONS AS FIELD.
C     OPTIONS ARE:
C        A) CINC.GT.0        USE INPUT VALUE OF AFAC AS SCALING
C                            FACTOR FOR PRINTING.
C                            USE INPUT VALUE OF CINC AS CONTOUR
C                            INTERVAL FOR PLOTTING.
C        B) CINC.LT.-1       FOR PRINTING NORMALISE MAX. ABS. VALUE
C                            OF FIELD TO 1000.
C                            FOR PLOTTING NORMALISE RANGE OF FIELD
C                            TO 1000. CONTOUR INTERVAL IS -CINC IN
C                            NORMALISED UNITS.
C        C) -1.LE.CINC.LT.0  FOR PRINTING NORMALISE MAX. ABS. VALUE
C                            OF FIELD TO LIE BETWEEN 0.1 AND 1 BY
C                            REMOVING POWERS OF 10 ONLY.
C                            FOR PLOTTING NORMALISE RANGE OF FIELD
C                            TO LIE BETWEEN 0.1 AND 1 BY REMOVING
C                            POWERS OF 10 ONLY. CONTOUR INTERVAL IS
C                            -CINC IN NORMALISED UNITS.
C     IFAIL VALUES ARE AS FOLLOWS:
C        -3    :    INPUT PRINT FACTOR & CONTOUR INTERVAL BOTH ZERO.
C        -2    :    INPUT PRINT FACTOR ZERO.
C        -1    :    INPUT CONTOUR INTERVAL ZERO.
C         0    :    SUCCESSFUL COMPLETION.
C         1    :    FIELD HAS ZERO RANGE :  NO CONTOURS SET UP.
C         2    :    TOO MANY CONTOURS REQUIRED : ONLY NCNT SET UP.
C
C
C----------------------------------------------------------------------
      INTEGER    IM,I,ILO,IEXP
      REAL*8     CLV,ZFRACT,ZA10,ZRANGE,ZAMAX
      INTEGER    ISMIN,ISMAX
C----------------------------------------------------------------------
      IFAIL=0
C
C     FIND MINIMUM AND MAXIMUM VALUES.
C
      IM=ISMAX(JXY,G,1)
      GMAX=G(IM)
      IM=ISMIN(JXY,G,1)
      GMIN=G(IM)
C
C     CHECK INPUT ARGUMENTS.
C
      IF (AFAC.EQ.0..AND.CINC.EQ.0.) THEN
         GNORM=0.
         GINC=0.
         ICNT=0
         IFAIL=-3
         RETURN
      ELSEIF (AFAC.EQ.0.) THEN
         GNORM=0.
         IFAIL=-2
      ENDIF
C
C     CHECK FOR ZERO RANGE.
C
      ZAMAX=MAX(ABS(GMAX),ABS(GMIN))
      ZRANGE=GMAX-GMIN
      IF (ZRANGE.LE.ZAMAX*1.E-10) THEN
         IF (AFAC.NE.0.) GNORM=1./AFAC
         GINC=0.
         ICNT=0
         IFAIL=1
         RETURN
      ENDIF
C
C     SET UP FACTOR FOR PRINTED OUTPUT AND CONTOUR LEVELS FOR PLOTTING.
C
      IF (CINC.GT.0) THEN
         IF (AFAC.NE.0.) GNORM=1./AFAC
         GINC=CINC
      ELSEIF (CINC.LT.-1.) THEN
         IF (AFAC.NE.0.) GNORM=1000./ZAMAX
         GINC=-ZRANGE*CINC/1000.
      ELSEIF (CINC.LT.0.) THEN
         IF (AFAC.NE.0.) THEN
            IEXP=LOG10(ZAMAX)*.99999
            IF (ZAMAX.GT.1.) IEXP=IEXP+1
            ZA10=10.**IEXP
            GNORM=1000./ZA10
         ENDIF
         IEXP=LOG10(ZRANGE)*.99999
         IF (ZRANGE.GT.1.) IEXP=IEXP+1
         ZA10=10.**IEXP
         ZFRACT=ZRANGE/ZA10
         IF (ZFRACT.GT.0.5) THEN
            GINC=-ZA10*CINC
         ELSEIF (ZFRACT.GT.0.2) THEN
            GINC=-ZA10*CINC*0.5
         ELSE
            GINC=-ZA10*CINC*0.2
         ENDIF
      ELSE
         IF (AFAC.NE.0.) GNORM=1./AFAC
         GINC=0.
         ICNT=0
         IFAIL=-1
         RETURN
      ENDIF
C
C     CALCULATE CONTOUR LEVELS.
C
      ILO=INT(GMIN/GINC)
      IF (GMIN.GE.0.) ILO=ILO+1
      CLV=FLOAT(ILO-1)*GINC
      I=0
 10   CLV=CLV+GINC
      I=I+1
      IF (I.GT.NCNT) GOTO 20
      IF (CLV.GT.GMAX) GOTO 30
      CL(I)=CLV
      GOTO 10
 20   IFAIL=2
 30   ICNT=I-1
C
      RETURN
      END
