C----------------------------------------------------------------------
      SUBROUTINE HANAL(GV,SV,NLS,ITYPE)
C----------------------------------------------------------------------
      IMPLICIT   NONE
      INTEGER    NLS,ITYPE
C
C     PERFORMS DIRECT LEGENDRE TRANSFORM FOR A (SET OF) FIELD(S)
C     HAVING A TOTAL OF NLS LEVELS, FROM FOURIER TO SPECTRAL SPACE.
C     THE TYPE OF FUNCTION AND THENCE OF TRANSFORM IS CHOSEN BY ITYPE
C     WITH THE FOLLOWING CURRENTLY DEFINED:
C        ITYPE=1,2   :   ALP     :   NORMAL TRANSFORM.
C        ITYPE=3,4   :   DALP    :   Y-DERIVATIVE.
C     AN EVEN/ODD VALUE OF ITYPE DENOTES A SPECTRAL FIELD OF EVEN/ODD
C     SYMMETRY.
C
      INCLUDE   'param1.inc'
      INCLUDE   'param2.inc'
      INCLUDE   'blank.inc'
      INCLUDE   'legau.inc'
      INCLUDE   'polyno.inc'
C
      COMPLEX*16   SV(IGA*NLS),GV(IGL*NLS),TEMP
      REAL*8       ALPN(MJP,JGL,4)
      EQUIVALENCE (ALPN(1,1,1),ALP(1,1))
C----------------------------------------------------------------------
      INTEGER      IALP,IW,IN,MP,IM,IP,INC,IHEM,IB,IV,IA,M,IGPAR,ISPAR
      REAL*8       AWT
C----------------------------------------------------------------------
C
 6900 FORMAT(/' ***ABORT : HANAL CALLED WITH TYPE =',I5)
C
C     USE ITYPE TO DEFINE TRANSFORM TYPE AND SYMMETRY LABELS.
C     ISPAR IS SYMMETRY OF SPECTRAL FIELD    = 0 FOR D,T,SP ETC.
C                                            = 1 FOR Z.
C     IGPAR IS SYMMETRY OF FOURIER FIELD: SAME AS ISPAR UNLESS TRANSFORM
C                                         INVOLVES A D/DY.
C
      IF (ITYPE.LE.0.OR.ITYPE.GE.5) THEN
         WRITE(6,6900) ITYPE
         CALL ABORT
      ENDIF
      IALP=(ITYPE+1)/2
      ISPAR=MOD(ITYPE,2)
      IGPAR=ISPAR
      IF (IALP.EQ.2) IGPAR=1-ISPAR
C
C     FOR A GLOBAL RUN, SUM AND DIFFERENCE THE COMPLETE FOURIER
C     TRANSFORMS AT THE NORTHERN AND SOUTHERN LATITUDE ROWS TO GIVE
C     THE EVEN AND ODD CONTRIBUTIONS.
C     SEPARATE CODE FOR EACH SYMMETRY:
C        IGPAR=0 : EVEN (IA) TO PRECEDE ODD (IB).
C        IGPAR=0 : ODD (IA) TO PRECEDE EVEN (IB).
C
      IF (NHEM.EQ.2) THEN
         IF (IGPAR.EQ.0) THEN
            DO 10 M=1,NWW
            IA=M-IGL
CDIR$ IVDEP
            DO 10 IV=1,NLS
            IA=IA+IGL
            IB=IA+IDL
            TEMP=GV(IA)
            GV(IA)=.5*(TEMP+GV(IB))
            GV(IB)=.5*(TEMP-GV(IB))
 10         CONTINUE
         ELSE
            DO 20 M=1,NWW
            IA=M-IGL
CDIR$ IVDEP
            DO 20 IV=1,NLS
            IA=IA+IGL
            IB=IA+IDL
            TEMP=GV(IA)
            GV(IA)=.5*(TEMP-GV(IB))
            GV(IB)=.5*(TEMP+GV(IB))
 20         CONTINUE
         ENDIF
      ENDIF
C
C     SET UP APPROPRIATE GAUSSIAN WEIGHT FOR CURRENT LATITUDE.
C     DEPENDS ON TRANSFORM TYPE.
C     ASSUMES JH IN /LEGAU/ CONTAINS LATITUDE COUNTER FROM CALLING LOOP.
C
      IF (IALP.EQ.1) AWT=AW(JH)*CSSQ(JH)
      IF (IALP.EQ.2) AWT=-AW(JH)
C
C     CALCULATE POLY ARRAY IN VECTOR LOOP BEFORE MAIN TRANSFORM.
C
      DO 30 IHEM=1,NHEM
      INC=(IHEM-1)*(1-ISPAR)+(2-IHEM)*ISPAR
      IA=INC-1
      DO 30 IP=1,NWJ2
      IA=IA+2
 30   POLY(IP,IHEM)=AWT*ALPN(IA,JL,IALP)
C
C     PERFORM DIRECT LEGENDRE TRANSFORM FROM THE EVEN AND ODD
C     PARTS OF THE FOURIER TRANSFORMS TO SPECTRAL SPACE.
C     SEPARATE CODE FOR NHEM=1,2 TO INCREASE EFFICIENCY.
C
      IF (NHEM.EQ.1) THEN
         IM=-IDL
         IP=-NWJ2
         DO 40 MP=1,MFP,MOCT
         IM=IM+1
         DO 40 IN=MP,NFP,MH
         IP=IP+1
         DO 40 IV=1,NLS
 40      SV(IP+IV*NWJ2)=SV(IP+IV*NWJ2) + POLY(IP+NWJ2,1)*GV(IM+IV*IDL)
      ELSE
         IM=-IDL
         IP=-NWJ2
         DO 50 MP=1,MFP,MOCT
         IM=IM+1
         DO 50 IN=MP,NFP,MH
         IP=IP+1
CDIR$    IVDEP
         DO 50 IV=1,NLS*NHEM,2
         IW=IV+1
         SV(IP+IV*NWJ2)=SV(IP+IV*NWJ2) + POLY(IP+NWJ2,1)*GV(IM+IV*IDL)
   50    SV(IP+IW*NWJ2)=SV(IP+IW*NWJ2) + POLY(IP+NWJ2,2)*GV(IM+IW*IDL)
      ENDIF
C
      RETURN
      END
