C----------------------------------------------------------------------
      SUBROUTINE HEXP(SV,GV,NLS,ITYPE)
C----------------------------------------------------------------------
      IMPLICIT   NONE
      INTEGER    NLS,ITYPE
C
C     PERFORMS INDIRECT LEGENDRE TRANSFORM FOR A (SET OF) FIELD(S)
C     HAVING A TOTAL OF NLS LEVELS, FROM SPECTRAL TO FOURIER SPACE.
C     THE TYPE OF FUNCTION AND THENCE OF TRANSFORM IS CHOSEN BY ITYPE
C     WITH THE FOLLOWING CURRENTLY DEFINED:
C        ITYPE=1,2   :   ALP     :   NORMAL TRANSFORM.
C        ITYPE=3,4   :   DALP    :   Y-DERIVATIVE.
C        ITYPE=5,6   :   RLP     :   DEL(-2).
C        ITYPE=7,8   :   RDLP    :   Y-DERIVATIVE OF DEL(-2).
C     AN EVEN/ODD VALUE OF ITYPE DENOTES A SPECTRAL FIELD OF EVEN/ODD
C     SYMMETRY.
C
      INCLUDE   'param1.inc'
      INCLUDE   'param2.inc'
      INCLUDE   'blank.inc'
      INCLUDE   'legau.inc'
      INCLUDE   'polyno.inc'
C
      COMPLEX*16   SV(IGA*NLS),GV(IGL*NLS),TEMP
      REAL*8       ALPN(MJP,JGL,4)
      EQUIVALENCE (ALPN(1,1,1),ALP(1,1))
C----------------------------------------------------------------------
      INTEGER   I,IB,M,IW,IV,IN,MP,IM,IP,IA,INC,IHEM,IGPAR,ISPAR,IALP
C----------------------------------------------------------------------
C
 6900 FORMAT(/' ***ABORT : HEXP CALLED WITH TYPE =',I5)
C
C
C     PRESET FOURIER ARRAY TO ZERO.
C
      DO 10 I=1,IGL*NLS
 10   GV(I)=0.
C
C     USE ITYPE TO DEFINE TRANSFORM TYPE AND SYMMETRY LABELS.
C     ISPAR IS SYMMETRY OF SPECTRAL FIELD    = 0 FOR D,T,SP ETC.
C                                            = 1 FOR Z.
C     IGPAR IS SYMMETRY OF FOURIER FIELD: SAME AS ISPAR UNLESS TRANSFORM
C                                         INVOLVES A D/DY.
C
      IF (ITYPE.LE.0.OR.ITYPE.GE.9) THEN
         WRITE(6,6900) ITYPE
         CALL ABORT
      ENDIF
      IALP=(ITYPE+1)/2
      ISPAR=MOD(ITYPE,2)
      IGPAR=ISPAR
      IF (IALP.EQ.2.OR.IALP.EQ.4) IGPAR=1-ISPAR
C
C     CALCULATE POLY ARRAY IN VECTOR LOOP BEFORE MAIN TRANSFORM.
C
      DO 30 IHEM=1,NHEM
      INC=(IHEM-1)*(1-ISPAR)+(2-IHEM)*ISPAR
      IA=INC-1
      DO 20 IP=1,NWJ2
      IA=IA+2
 20   POLY(IP,IHEM)=ALPN(IA,JL,IALP)
 30   CONTINUE
C
C     PERFORM INVERSE LEGENDRE TRANSFORM FROM SPECTRAL SPACE TO FORM
C     THE EVEN AND ODD CONTRIBUTIONS TO THE FOURIER TRANSFORMS.
C     SEPARATE CODE FOR NHEM=1,2 TO INCREASE EFFICIENCY.
C
      IF (NHEM.EQ.1) THEN
         IM=-IDL
         IP=-NWJ2
         DO 40 MP=1,MFP,MOCT
         IM=IM+1
         DO 40 IN=MP,NFP,MH
         IP=IP+1
         DO 40 IV=1,NLS
 40      GV(IM+IV*IDL)=GV(IM+IV*IDL) + POLY(IP+NWJ2,1)*SV(IP+IV*NWJ2)
      ELSE
         IM=-IDL
         IP=-NWJ2
         DO 50 MP=1,MFP,MOCT
         IM=IM+1
         DO 50 IN=MP,NFP,MH
         IP=IP+1
CDIR$    IVDEP
         DO 50 IV=1,NLS*NHEM,2
         IW=IV+1
         GV(IM+IV*IDL)=GV(IM+IV*IDL) + POLY(IP+NWJ2,1)*SV(IP+IV*NWJ2)
 50      GV(IM+IW*IDL)=GV(IM+IW*IDL) + POLY(IP+NWJ2,2)*SV(IP+IW*NWJ2)
      ENDIF
C
C     FOR A GLOBAL RUN, SUM AND DIFFERENCE EVEN AND ODD CONTRIBUTIONS
C     TO GIVE THE COMPLETE FOURIER TRANSFORMS AT THE NORTHERN AND
C     SOUTHERN LATITUDE ROWS.  SEPARATE CODE FOR EACH SYMMETRY:
C        IGPAR=0 : EVEN (IA) PRECEDES ODD (IB).
C        IGPAR=0 : ODD (IA) PRECEDES EVEN (IB).
C
      IF (NHEM.EQ.2) THEN
         IF (IGPAR.EQ.0) THEN
            DO 60 M=1,NWW
            IA=M-IGL
CDIR$ IVDEP
            DO 60 IV=1,NLS
            IA=IA+IGL
            IB=IA+IDL
            TEMP=GV(IA)
            GV(IA)=TEMP+GV(IB)
            GV(IB)=TEMP-GV(IB)
 60         CONTINUE
         ELSE
            DO 70 M=1,NWW
            IA=M-IGL
CDIR$ IVDEP
            DO 70 IV=1,NLS
            IA=IA+IGL
            IB=IA+IDL
            TEMP=GV(IA)
            GV(IA)=GV(IB)+TEMP
            GV(IB)=GV(IB)-TEMP
 70         CONTINUE
         ENDIF
      ENDIF
C
      RETURN
      END
