C----------------------------------------------------------------------
      SUBROUTINE INIMSK
C----------------------------------------------------------------------
      IMPLICIT   NONE
C
C     INITIALISE OROGRAPHIC MASK FIELDS FOR CURRENT ANALYSIS TIME.
C     WRITE FIELDS TO DATA TRANSFER FILE (UTF) IF REQUIRED.
C
      INCLUDE   'param1.inc'
      INCLUDE   'param2.inc'
      INCLUDE   'bats.inc'
      INCLUDE   'blank.inc'
      INCLUDE   'comioc.inc'
      INCLUDE   'comiol.inc'
      INCLUDE   'commsk.inc'
      INCLUDE   'comprl.inc'
      INCLUDE   'comrog.inc'
      INCLUDE   'outcon.inc'
      INCLUDE   'gridp.inc'
C
      REAL*8       P2MSK(MGP,JGGP)
      EQUIVALENCE (P2MSK(1,1),UG(1))
C
      REAL*8       P3MIN(JGG),P3AV(JGG)
      EQUIVALENCE (P3MIN(1),P3MSK(1,1)),(P3AV(1),P3MSK(1,2))
      LOGICAL      LOM2,LOM3
      CHARACTER*25 MNAME2,MNAME3
C
      REAL*8       ZERO
      PARAMETER   (ZERO=0.)
C----------------------------------------------------------------------
      INTEGER      IGMX,I,J,NC,JS,IHEM,JJ,JZ,IM,ICNT,IFAIL
      REAL*8       DUM,CL,GNORM,GINC,P2MAX
      INTEGER      ISMIN
      REAL*8       SSUM
C----------------------------------------------------------------------
      DATA MNAME2/'MASK2 SURFACE PRESSURE   '/
      DATA MNAME3/'MASK3 MIN AND AV PRESSURE'/
      SAVE MNAME2,MNAME3
C
 6000 FORMAT(/' ***ABORT IN INIMSK: 2-D MASK ARRAY TOO LARGE FOR'
     >       ,' EQUIVALENCING IN GRIDP: IDH, IGMX =',2I10)
C
C     CHECK COMMON GRIDP IS LONG ENOUGH FOR EQUIVALENCED MASK ARRAY
C     MUST OCCUPY ONLY FROM UG TO PMSLG, TO AVOID ISENTROPIC ARRAYS.
C***  NB.  THIS CALCULATION DEPENDS ON THE SIZE/ORDERING OF GRIDP.
C
      IGMX=(14*NL+4)*IGC
      IF (IDH.GT.IGMX) THEN
         WRITE(6,6000) IDH,IGMX
         CALL ABORT
      ENDIF
C
C     PRESET FULL SURFACE PRESSURE MASK ARRAY TO ZERO.
C     SET MASK SWITCHES FOR THIS ANALYSIS TIME.  RETURN IF NO MASKS.
C
      DO 10 J=1,JGGP
      DO 10 I=1,MGP
 10   P2MSK(I,J)=ZERO
      LOM2=LMSK2.AND.(LOUTA.OR.LOUTF)
      LOM3=LMSK3.AND.(LOUTA.OR.LOUTF)
      IF (.NOT.(LOM2.OR.LOM3)) RETURN
C
C     READ SURFACE PRESSURE FIELD DIRECTLY INTO LAT-LONG MASK ARRAY.
C
      NC=NCGP(11)
      REWIND NC
      DO 20 J=1,JG
      IF (NHEM.EQ.1) THEN
         READ(NC) (P2MSK(I,J),I=1,MG)
      ELSE
         JS=JGGP+1-J
         READ(NC) (P2MSK(I,J),I=1,MG),(DUM,I=1,2),(P2MSK(I,JS),I=1,MG)
      ENDIF
 20   CONTINUE
      REWIND NC
C
C     NORMALISE SO THAT 0 -> P0 BECOMES 0 -> 1000.
C     OBTAIN ZONAL MEAN AND MINIMUM SURFACE PRESSURE AT EACH LATITUDE.
C     USE TO FORM REMAINING MASK FIELDS.
C
      PMFAC=1000.
      DO 50 IHEM=1,NHEM
      DO 50 J=1,JG
      JJ=(2-IHEM)*J + (IHEM-1)*(JGGP+1-J)
      JZ=JJ-(IHEM-1)
      DO 40 I=1,MG
 40   P2MSK(I,JJ)=PMFAC*P2MSK(I,JJ)
      P3AV(JZ)=SSUM(MG,P2MSK(1,JJ),1)*RMG
      IM=ISMIN(MG,P2MSK(1,JJ),1)
      P3MIN(JZ)=P2MSK(IM,JJ)
 50   CONTINUE
C
C     LAT-LONG MASK: PREPARE AND WRITE TO UTF IF REQUIRED.
C     CHECK THAT LOWEST PRESSURE LEVEL INTERSECTS OROGRAPHY.
C
      IF (LOM2) THEN
         CALL GVPREP(P2MSK,ZERO,ZERO,CL,1,GNORM,
     >               GINC,ICNT,P2MIN,P2MAX,IFAIL)
         IF (((PPR(NLPR)*PMFAC).GT.P2MIN).AND.LOUTA) THEN
            IF (LNETCDF) THEN
               CALL WTFCDF(P2MSK,MGP,JGGP,IZGTYP,0,3,2,1,1
     >              ,P2MIN,P2MAX,ZERO,DAY,DOY,MNAME2,25,NPCA,IFAIL)
            ELSE
               CALL WTFUTF(P2MSK,MGP,JGGP,IZGTYP,0,3,2,1,1
     >              ,P2MIN,P2MAX,ZERO,DAY,DOY,MNAME2,25,NPCA,IFAIL)
            ENDIF
         ENDIF
      ENDIF
C
C     ZONAL MEAN MASK: WRITE TO CDF/UTF IF REQUIRED.
C
      IF (LOM3.AND.LOUTA) THEN
         IF (LNETCDF) THEN
            CALL WTFCDF(P3MSK,JGG,2,11,0,4,2,1,1
     >           ,ZERO,ZERO,ZERO,DAY,DOY,MNAME3,25,NPCA,IFAIL)
         ELSE
            CALL WTFUTF(P3MSK,JGG,2,11,0,4,2,1,1
     >           ,ZERO,ZERO,ZERO,DAY,DOY,MNAME3,25,NPCA,IFAIL)
         ENDIF
      ENDIF
C
      RETURN
      END
