C----------------------------------------------------------------------
      SUBROUTINE INTER(GIN,GOUT,NLOUT,INTRL,FINTR,JOW,JXUP,JXDN)
C----------------------------------------------------------------------
      IMPLICIT   NONE
      INTEGER    NLOUT,JOW,JXUP,JXDN
C
C     INTERPOLATE VERTICALLY FROM INPUT ARRAY GIN(..,NL) TO OUTPUT
C     ARRAY GOUT(..,NLOUT), USING PRE-DETERMINED FACTORS FINTR AND
C     LEVELS INTRL(..,L) AND INTRL(..,L+1) IN INPUT DATA FOR LEVEL
C     L IN OUTPUT DATA.
C     IF JOW=1, INTERPOLATED DATA IS OVERWRITTEN ON FIRST NLOUT LEVELS
C     OF INPUT ARRAY.
C     EXTRAPOLATION BEYOND EXTREME MODEL LEVELS CONTROLLED BY NAMELIST
C     LOGICALS LXTRP2 AND LXTRP3 FOR LPC2 AND LPC3 FIELDS RESPECTIVELY.
C     IF SWITCHED ON, JXUP AND JXDN DEFINE OPTIONS USED.  IF SWITCHED
C     OFF, INPUT OPTIONS ARE OVERRIDDEN AND VALUES HELD CONSTANT BEYOND
C     EXTREME MODEL LEVELS (EQUIVALENT TO JXUP=JXDN=0).
C     CURRENT EXTRAPOLATION OPTIONS ARE:
C        UPWARD  : JXUP = 0  :  CONSTANT VALUE ABOVE PSG(,1)
C                         1  :  SAME AS INTERP - LINEAR IN LN(P)
C                         2  :  LINEAR INTERP IN P, TO ZERO AT P=0
C        DOWNWARD: JXDN = 0  :  CONSTANT VALUE BELOW PSG(,NL)
C                         1  :  SAME AS INTERP - LINEAR IN LN(P)
C                         2  :  INTERP TO SURFACE OMEGA, CONST BELOW
C                         3  :  TEMPERATURE: CONST LAPSE BELOW PSG(,NL)
C                         4  :  GEOPOTENTIAL: BASED ON CONSTANT LAPSE
C                         5  :  CONSTANT DOWN TO SURFACE, ZERO BELOW
C     NB.  COMMON COMGRM MUST NOT BE CALLED AS IT CONTAINS INTRL.
C     NB.  COMMON  GRIDP MUST NOT BE CALLED AS IT CAN CONTAIN GIN.
C
      INCLUDE   'param1.inc'
      INCLUDE   'param2.inc'
      INCLUDE   'comprl.inc'
      INCLUDE   'comrog.inc'
      INCLUDE   'legau.inc'
C
      LOGICAL    LX
      INTEGER    INTRL(IGC,NLOUT)
      REAL*8     GIN(IGC,NL),GOUT(IGC,NLOUT),FINTR(IGC,NLOUT)
C----------------------------------------------------------------------
      INTEGER    IXUP,IXDN,IHEM,IOF,N,I,II,LM
C----------------------------------------------------------------------
C
 6900 FORMAT(/' ***ABORT IN INTER: IXUP = ',I3,' NOT ALLOWED')
 6910 FORMAT(/' ***ABORT IN INTER: IXDN = ',I3,' NOT ALLOWED')
C
C     SET EXTRAPOLATION SWITCHES.  OVERRIDE INPUT OPTIONS IF NO EXTRAP.
C
      LX=(LINTP2.AND.LXTRP2).OR.(LINTP3.AND.LXTRP3)
      IXUP=JXUP
      IXDN=JXDN
      IF (.NOT.LX) THEN
         IXUP=0
         IXDN=0
      ENDIF
C
      IF (LX.AND.(IXUP.LT.0.OR.IXUP.GT.2)) THEN
         WRITE(6,6900) IXUP
         CALL ABORT
      ENDIF
      IF (LX.AND.(IXDN.LT.0.OR.IXDN.GT.5)) THEN
         WRITE(6,6910) IXDN
         CALL ABORT
      ENDIF
C
      DO 300 IHEM=1,NHEM
      IOF=(IHEM-1)*MGPP
C
C     FIRST PERFORM INTERPOLATION AND IMPLIED EXTRAPOLATION USING
C     DEFAULT FACTORS.
C
      DO 5 N=1,NLOUT
      DO 5 I=1,MG
      II=I+IOF
      LM=INTRL(II,N)
 5    GOUT(II,N)=GIN(II,LM)+FINTR(II,N)*(GIN(II,LM+1)-GIN(II,LM))
C
C     UPWARD EXTRAPOLATION.  SKIP LEVELS FOR WHICH PRESSURE LEVEL IS
C     WHOLLY BELOW TOP MODEL LEVEL.
C
      IF (IXUP.EQ.1) GOTO 100
      DO 90 N=1,NLOUT
      IF (PPR(N).GE.PSGMX(1,IHEM)) GOTO 90
      IF (IXUP.EQ.0) THEN
         DO 10 I=1,MG
         II=I+IOF
         IF (PPR(N).LT.PSG(II,1)) GOUT(II,N)=GIN(II,1)
 10      CONTINUE
      ELSEIF (IXUP.EQ.2) THEN
         DO 20 I=1,MG
         II=I+IOF
         IF (PPR(N).LT.PSG(II,1))  GOUT(II,N)=GIN(II,1)*PPR(N)/PSG(II,1)
 20      CONTINUE
      ENDIF
 90   CONTINUE
C
C     DOWNWARD EXTRAPOLATION.  SKIP LEVELS FOR WHICH PRESSURE LEVEL IS
C     WHOLLY ABOVE BOTTOM MODEL LEVEL.
C
 100  CONTINUE
      IF (IXDN.EQ.1) GOTO 200
      DO 190 N=1,NLOUT
      IF (PPR(N).LE.PSGMN(NL,IHEM)) GOTO 190
      IF (IXDN.EQ.0) THEN
         DO 110 I=1,MG
         II=I+IOF
         IF (PPR(N).GT.PSG(II,NL)) GOUT(II,N)=GIN(II,NL)
 110     CONTINUE
      ELSEIF (IXDN.EQ.2) THEN
         DO 120 I=1,MG
         II=I+IOF
         IF (PPR(N).GT.PSRF(II)) GOUT(II,N)=OMSRF(II)
         IF (PPR(N).GT.PSG(II,NL).AND.PPR(N).LE.PSRF(II))
     >      GOUT(II,N)=GIN(II,NL)+(OMSRF(II)-GIN(II,NL))*
     >                 (PPR(N)-PSG(II,NL))/(PSRF(II)-PSG(II,NL))
 120     CONTINUE
      ELSEIF (IXDN.EQ.3) THEN
         DO 130 I=1,MG
         II=I+IOF
         IF (PPR(N).GT.PSG(II,NL))
     >      GOUT(II,N)=GIN(II,NL)*((PPR(N)/PSG(II,NL))**XLAPSE)
 130     CONTINUE
      ELSEIF (IXDN.EQ.4) THEN
         DO 140 I=1,MG
         II=I+IOF
         IF (PPR(N).GT.PSRF(II))
     >      GOUT(II,N)=GSG(II,JL)-(TSRF(II)/XLAPSE)*
     >                            ((PPR(N)/PSRF(II))**XLAPSE-1.)
         IF (PPR(N).GT.PSG(II,NL).AND.PPR(N).LE.PSRF(II))
     >      GOUT(II,N)=GIN(II,NL)+(GSG(II,JL)-GIN(II,NL))*
     >               LOG(PPR(N)/PSG(II,NL))/LOG(PSRF(II)/PSG(II,NL))
 140     CONTINUE
      ELSEIF (IXDN.EQ.5) THEN
         DO 150 I=1,MG
         II=I+IOF
         IF (PPR(N).GT.PSRF(II)) GOUT(II,N)=0.
         IF (PPR(N).GT.PSG(II,NL).AND.PPR(N).LE.PSRF(II))
     >      GOUT(II,N)=GIN(II,NL)
 150     CONTINUE
      ENDIF
 190  CONTINUE
C
C     COPY TO OVERWRITE THE INPUT ARRAY IF REQUESTED.
C
 200  CONTINUE
      IF (JOW.EQ.1) THEN
         DO 210 N=1,NLOUT
         DO 210 I=1,MG
         II=I+IOF
 210     GIN(II,N)=GOUT(II,N)
      ENDIF
C
 300  CONTINUE
C
      RETURN
      END
