C----------------------------------------------------------------------
      SUBROUTINE LGNDRE
C----------------------------------------------------------------------
      IMPLICIT   NONE
C
C     CALCULATE ASSOCIATED LEGENDRE FUNCTIONS AT CURRENT LATITUDE
C     IN ARRAY ALP AND THEIR MERIDIONAL DERIVATIVES IN ARRAY DALP.
C          ALP = PNM,     DALP = (1-MU**2)DPNM/DMU.
C     SPECTRAL TRUNCATION IS JAGGED TRIANGULAR AT TOTAL WAVENUMBER
C     NN (DEFINED IN PARAMETER STATEMENT), WITH EQUAL NUMBERS OF
C     EVEN AND ODD FUNCTIONS FOR EACH ZONAL WAVENUMBER.
C     INCLUDES ZONAL WAVENUMBERS 0, MOCT, 2*MOCT,... TO MAXIMUM NN-1.
C     USES LATITUDE COUNTER JH IN COMMON LEGAU AS THE SUBSCRIPT
C     OF THE SI ARRAY FOR MU=SIN(LAT) AND CS FOR COS(LAT).
C
      INCLUDE   'param1.inc'
      INCLUDE   'param2.inc'
      INCLUDE   'blank.inc'
      INCLUDE   'legau.inc'
C----------------------------------------------------------------------
      INTEGER   LM,M1,M,MMO,M2,JFM,K,N
      REAL*8    F1M,AM,A2M,E2,F2M,AMSQ,AN,AN2,ANM2,E1
C----------------------------------------------------------------------
      LM=2
      ALP(1,JL)=SQRT(.5)
      F1M=SQRT(1.5)
      ALP(2,JL)=F1M*SI(JH)
      DALP(1,JL)=0.
      DO 40 M1=1,MFP
      M=M1-1
      AM=M
      A2M=M+M
      E2=SQRT(A2M+3.)
      IF (M.EQ.0) GOTO 10
      F2M=-F1M*CS(JH)/SQRT(A2M)
      F1M=F2M*E2
      IF (M.NE.MMO) GOTO 40
      LM=LM+1
      ALP(LM,JL)=F2M
      LM=LM+1
      ALP(LM,JL)=F1M*SI(JH)
      DALP(LM-1,JL)=-AM*ALP(LM,JL)/E2
 10   M2=M+2
      MMO=M+MOCT
      JFM=((NFP-M1)/2)*2+M2-1
      IF (JFM.LT.M2) GOTO 30
      K=LM-M2+1
      AMSQ=AM*AM
      DO 20 N=M2,JFM
      AN=N
      AN2=N*N
      ANM2=(N-1)*(N-1)
      E1=SQRT((ANM2-AMSQ)/(4.*ANM2-1.))
      E2=SQRT((4.*AN2-1.)/(AN2-AMSQ))
      ALP(K+N,JL)=E2*(SI(JH)*ALP(K+N-1,JL)-E1*ALP(K+N-2,JL))
      DALP(K+N-1,JL)= (1.-AN)*ALP(K+N,JL)/E2+AN*E1*ALP(K+N-2,JL)
 20   CONTINUE
      LM=LM+JFM-M2+1
 30   CONTINUE
      DALP(LM,JL)=-AN*SI(JH)*ALP(LM,JL)+(AN+AN+1.)*ALP(LM-1,JL)/E2
 40   CONTINUE
C
      RETURN
      END
