C----------------------------------------------------------------------
      SUBROUTINE NTEND
C----------------------------------------------------------------------
      IMPLICIT   NONE
C
C     CALCULATE NON-LINEAR MOMENTUM TENDENCIES FROM BOUNDARY LAYER AND
C     VERTICAL DIFFUSION SCHEMES, IF INCLUDED.  USED IN GDIAG FOR
C     GLOBAL ENERGETICS, PLUS FIELDS MAY ALSO BE PRINTED/PLOTTED.
C     APPLIES PARAMETERISATION SCHEMES FROM THE SIMPLE GCM (SGCM)
C     PDN=BGCM5,ID=UPLIB,ED=5 TO THE HISTORY FILE DATA AT THE CURRENT
C     ANALYSIS TIME.  NOTE THAT THE TENDENCIES IN THE MODEL ARE SLIGHTLY
C     DIFFERENT SINCE THEY ARE COMPUTED IN A SPLIT TIMESTEP THERE.
C
      INCLUDE   'param1.inc'
      INCLUDE   'param2.inc'
      INCLUDE   'bats.inc'
      INCLUDE   'blank.inc'
      INCLUDE   'gridp.inc'
      INCLUDE   'legau.inc'
      INCLUDE   'phys.inc'
C
      REAL*8     FA(NL),BVP(MG),BVM(MG)
C----------------------------------------------------------------------
      INTEGER    I,IHEM,JOF,J,L,IOFM,JP,JM
      REAL*8     BLCONJ,VM,CMODV,TLPH,FTSQ
C----------------------------------------------------------------------
C
C     ZERO PRESETS.  NO CALCULATIONS IF PARAMETERISATIONS NOT SWITCHED
C     ON AT CURRENT ANALYSIS TIME.
C
      DO 10 I=1,IGD
      FUG(I)=0.0
      FVG(I)=0.0
 10   CONTINUE
      IF (KOUNT.LE.KBEGP) RETURN
C
C     BOUNDARY LAYER SCHEME.
C     MOMENTUM TENDENCIES AT LOWEST MODEL LEVEL ASSOCIATED WITH SURFACE
C     FLUXES.  BULK AERODYNAMIC FORMULA WITHOUT STABILITY DEPENDENCE.
C
      IF (LBL) THEN
         DO 30 IHEM=1,NHEM
         JOF=NLM*IGC + (IHEM-1)*MGPP
         DO 30 I=1,MG
         J=JOF+I
         BLCONJ=BLCON/TSTAR(J,JH)
         VM=SQRT(SECSQ(JH)*(UG(J)*UG(J)+VG(J)*VG(J)))
         CMODV=BLCONJ*(VM+BLVAD)
         FUG(J)=FUG(J)-CMODV*UG(J)
         FVG(J)=FVG(J)-CMODV*VG(J)
 30      CONTINUE
      ENDIF
C
C     VERTICAL DIFFUSION SCHEME.
C     LINEAR DIFFUSIVE FLUX THROUGHOUT MODEL DEPTH WITH CONSTANT COEFF.
C
      IF (LVD) THEN
      DO 100 L=1,NL
 100  FA(L)=1.0/DSIGMA(L)
      DO 140 IHEM=1,NHEM
      IOFM=(IHEM-1)*MGPP
      DO 110 I=1,MG
      J=I+IOFM
      JP=J+IGC
      TLPH=TG(JP)+TG(J)
      FTSQ=FB(1)/(TLPH*TLPH)
      BVP(I)=AKVV*FTSQ
      FUG(J)=FUG(J)+FA(1)*BVP(I)*(UG(JP)-UG(J))
      FVG(J)=FVG(J)+FA(1)*BVP(I)*(VG(JP)-VG(J))
 110  CONTINUE
      DO 120 L=2,NLM
      JOF=(L-1)*IGC+IOFM
      DO 120 I=1,MG
      J=JOF+I
      JP=J+IGC
      JM=J-IGC
      BVM(I)=BVP(I)
      TLPH=TG(JP)+TG(J)
      FTSQ=FB(L)/(TLPH*TLPH)
      BVP(I)=AKVV*FTSQ
      FUG(J)=FUG(J)+FA(L)*(BVP(I)*(UG(JP)-UG(J))-BVM(I)*(UG(J)-UG(JM)))
      FVG(J)=FVG(J)+FA(L)*(BVP(I)*(VG(JP)-VG(J))-BVM(I)*(VG(J)-VG(JM)))
 120  CONTINUE
      JOF=NLM*IGC+IOFM
      DO 130 I=1,MG
      J=JOF+I
      JM=J-IGC
      FUG(J)=FUG(J)-FA(NL)*BVP(I)*(UG(J)-UG(JM))
      FVG(J)=FVG(J)-FA(NL)*BVP(I)*(VG(J)-VG(JM))
 130  CONTINUE
 140  CONTINUE
      ENDIF
C
      RETURN
      END
