C----------------------------------------------------------------------
      SUBROUTINE PHYSAV
C----------------------------------------------------------------------
      IMPLICIT   NONE
C
C     PROCESS TIME SERIES AND TIME AVERAGES OF GLOBALLY AVERAGED
C     PARAMETERISATION DIAGNOSTICS.  THESE HAVE BEEN ACCUMULATED
C     ON CHANNEL NCGDG BY ROUTINE PHYSOP AT EACH ANALYSIS TIME.
C     PHMN(K,1) CONTAINS INSTANTANEOUS VALUES.
C     PHMN(K,2) CONTAINS VALUES AVERAGED OVER PREVIOUS KOUNTP STEPS.
C     PHMN(K,3) CONTAINS TIME AVERAGES OVER ENTIRE RUN USING THE
C               PHMN(K,2) VALUES, SO OMITTING KSTART.
C     NB.  TIME AVERAGES ARE FOR PERIOD KSTART TO KEND, IRRESPECTIVE
C     OF WHETHER PARAMETERISATIONS ACTIVE FROM KBEGP > KSTART.
C     NB.  PHYSAV MUST ONLY BE CALLED AFTER FINAL CALL TO GDIAG, SINCE
C     THE TIME SERIES ARRAY ADIAG IS EQUIVALENCED TO AN ARRAY IN
C     COMMON SERIES USED BY GDIAG.
C
      INCLUDE   'param1.inc'
      INCLUDE   'param2.inc'
      INCLUDE   'bats.inc'
      INCLUDE   'blank.inc'
      INCLUDE   'comioc.inc'
      INCLUDE   'comiol.inc'
      INCLUDE   'outcon.inc'
      INCLUDE   'phys.inc'
      INCLUDE   'series.inc'
C
      REAL*8      PHMN(NPH,3),ADIAG(NPMAX9,8),ADMX(3),ADMN(3)
      INTEGER     NI(3),IYD(3)
      CHARACTER   IXLAB*22,IYLAB(3)*30,IDAN(8)*3,LTEMP*8,TIMANN(3)*7
      EQUIVALENCE (ADIAG(1,1),ACONV(1,1))
C
C----------------------------------------------------------------------
      INTEGER     I,K,NPTS9,JOUNT,N,NMIN,NMAX,IANC,IXT,IYT,I2,NPLOT,I1
      INTEGER     NBEG,NEND,IXI
      REAL*8      RTOF,TDAY,SSBL,SHBL,SLBL,RRCR,RRLR,RAIN,PREC,EVAP
      REAL*8      XU,YL,XMN,XL
C----------------------------------------------------------------------
      DATA NI/1,3,4/,IYD/10,20,10/,ADMX/1.,200.,10./,ADMN/3*0./
      DATA IXLAB/'    TIME IN DAYS*.'/
      DATA IYLAB/'         SURFACE STRESS N/M2*.'
     :,          '         SURFACE FLUXES W/M2*.'
     :,          '         PRECIP/EVAP MM/DAY*.'
     :/
      DATA LTEMP/'      *.'/
      DATA IDAN /'TAU','  H','  L','  R',' CR',' LR','  R','  E'/
      DATA TIMANN/'(INST) ','(AVER) ',' (TAV) '/
C
 6000 FORMAT(' GLOBAL SURFACE DIAGNOSTICS AT DAY ',F6.2)
 6010 FORMAT(' TIME AVERAGE GLOBAL SURFACE DIAGNOSTICS')
 6020 FORMAT(1P,5X,A,'  HFLUX=',E12.5,'  LFLUX=',E12.5
     :              ,' PRECIP=',E12.5,' W/M2   STRESS=',E12.5,' N/M2')
 6030 FORMAT(1P,5X,A,'  CRAIN=',E12.5,'  LRAIN=',E12.5
     :              ,'   RAIN=',E12.5,'   EVAP=',E12.5,' MM/DAY')
 6040 FORMAT(' ***PHYSAV : TIME SERIES OF PHYSICS GLOBAL AVERAGES'
     :      ,' ON PLOT FILE STOPS AT DAY',F10.2)
C
C     WRITE HEADER.  PRESET TIME AVERAGES AND SET CONSTANTS.
C
      IF (NPC9.NE.6) WRITE(NPC9,'(/////)')
      WRITE(6,'(/////)')
      IF (LTAV) THEN
         DO 10 K=1,NPH
 10      PHMN(K,3)=0.
      ENDIF
      RTOF=WW*CLATNT*.001*1000./PI2
      REWIND NCGDG
      NPTS9=0
C
C     ----------------------------------------------------------------
C     MAIN LOOP OVER ANALYSIS TIMES.  FOR JOUNT >= 0 READ DATA FROM
C     CHANNEL NCGDG.  JOUNT<0 DENOTES END OF SERIES SO PROCESS AVERAGES.
C
 20   READ(NCGDG) JOUNT,TDAY
      IF (JOUNT.EQ.-999.AND..NOT.LTAV) RETURN
C
      IF (JOUNT.GE.0) THEN
         IF (NPC9.NE.6) WRITE(NPC9,6000) TDAY
         WRITE(6,6000) TDAY
         READ(NCGDG) ((PHMN(K,N),K=1,NPH),N=1,2)
         IF (LTAV.AND.JOUNT.GT.KSTART) THEN
            DO 30 K=1,NPH
 30         PHMN(K,3)=PHMN(K,3)+FTAV*PHMN(K,2)
         ENDIF
         NMIN=1
         NMAX=2
      ELSE
         IF (NPC9.NE.6) WRITE(NPC9,6010)
         WRITE(6,6010)
         NMIN=3
         NMAX=3
      ENDIF
C
      DO 40 N=NMIN,NMAX
      SSBL=PHMN(1,N)
      SHBL=PHMN(2,N)
      SLBL=PHMN(3,N)
      RRCR=PHMN(4,N)
      RRLR=PHMN(5,N)
      RAIN=PHMN(6,N)
      PREC=RAIN*RTOF
      EVAP=SLBL/RTOF
      IF (NPC9.NE.6) THEN
         WRITE(NPC9,6020) TIMANN(N),SHBL,SLBL,PREC,SSBL
         WRITE(NPC9,6030) TIMANN(N),RRCR,RRLR,RAIN,EVAP
      ENDIF
      WRITE(6,6020) TIMANN(N),SHBL,SLBL,PREC,SSBL
      WRITE(6,6030) TIMANN(N),RRCR,RRLR,RAIN,EVAP
 40   CONTINUE
      IF (JOUNT.LT.0) GOTO 50
C
C     COPY VALUES INTO TIME SERIES ARRAY TO BE SORTED AND COPIED TO
C     PLOT FILE BELOW.
C
      IF (LPHCOP.AND.NPTS9.LT.NPMAX9) THEN
         NPTS9=NPTS9+1
         IF (NPTS9.EQ.NPMAX9) WRITE(6,6040) DAY
         ADAY(NPTS9)=DAY
         ADIAG(NPTS9,1)=PHMN(1,2)
         ADIAG(NPTS9,2)=PHMN(2,2)
         ADIAG(NPTS9,3)=PHMN(3,2)
         ADIAG(NPTS9,5)=PHMN(4,2)
         ADIAG(NPTS9,6)=PHMN(5,2)
         ADIAG(NPTS9,7)=PHMN(6,2)
         ADIAG(NPTS9,4)=ADIAG(NPTS9,7)*RTOF
         ADIAG(NPTS9,8)=ADIAG(NPTS9,3)/RTOF
      ENDIF
C
      GOTO 20
C
C     ----------------------------------------------------------------
C     END OF LOOP OVER ANALYSIS TIMES.
C     WRITE TIME SERIES TO FILE FOR PLOTTING IN SUBSEQUENT JOB.
C
 50   IF (.NOT.LPHCOP) RETURN
      IANC=-1
      XU=XUEN
      YL=335.
      IXT=3
      IYT=3
      I2=0
      DO 100 NPLOT=1,3
      I1=I2+1
      I2=I1+NI(NPLOT)-1
      WRITE(IYLAB(NPLOT)(1:8),'(F8.3)')RNTAPE
      NBEG=1
      NEND=NBEG+ISPLIT
 80   IF (NEND.GT.NPTS9) NEND=NPTS9
      XMN=ADAY(NBEG)
      IF (NINT(XMN).EQ.1+NINT(KBEGP/TSPD)) XMN=0.
      XL=ADAY(NEND)-XMN
      IXI=NINT(XL)
      XL=XL/XU
      N=NEND-NBEG+1
      LNPIC=.TRUE.
      DO 90 I=I1,I2
      LTEMP(4:6)=IDAN(I)
      WRITE(NCPLOT(1)) N,IXT,IYT,LTEMP,IANC,IXLAB,IYLAB(NPLOT),XL,XU
     :         ,XMN,IXI,YL,ADMX(NPLOT),ADMN(NPLOT),IYD(NPLOT),LNPIC
      WRITE(NCPLOT(1)) (ADAY(K),K=NBEG,NEND),(ADIAG(K,I),K=NBEG,NEND)
 90   LNPIC=.FALSE.
      NBEG=NEND
      NEND=NBEG+ISPLIT
      IF (NBEG.LT.NPTS9) GOTO 80
 100  CONTINUE
C
      RETURN
      END
