C----------------------------------------------------------------------
      SUBROUTINE PHYSOP
C----------------------------------------------------------------------
      IMPLICIT   NONE
C
C     PROCESS 2-D SURFACE FIELDS OF PARAMETERISATION DIAGNOSTICS
C     AT CURRENT ANALYSIS TIME.  CONTAINED ON JG RECORDS OF HISTORY
C     FILE.  SKIP AND RETURN IF DIAGNOSTICS NOT REQUIRED.
C
C     (NPH-1) INSTANTANEOUS AND DAILY-AVERAGED FIELDS ARE READ, WITH
C     CONVECTIVE AND LARGE-SCALE RAINFALL RATES LAST.  THESE ARE ADDED
C     FOR THE TOTAL RAINFALL IN THE (NPH)TH FIELD.
C        NT=1 : INSTANTANEOUS FIELDS.
C        NT=2 : FIELDS AVERAGED OVER PRECEDING KOUNTP STEPS IN MODEL.
C
C     NB.  COMMON SPECTR IS USED AS WORKSPACE FOR GPV ARRAY.
C     NB.  CURRENT CODE EQUIVALENCES BOTH THE UNUSED VECTOR ARRAYS GX
C     AND GY TO GVE( ,3) IN COMMON DUM, SINCE GVE IS ONLY 3 FIELDS LONG.
C
      INCLUDE   'param1.inc'
      INCLUDE   'param2.inc'
      INCLUDE   'bats.inc'
      INCLUDE   'blank.inc'
      INTEGER    NCNT,NCR,NLR,NCRR,NLRR
      PARAMETER (NCNT=50,NCR=NPH-2,NLR=NPH-1)
      PARAMETER (NCRR=4,NLRR=5)
      INCLUDE   'comioc.inc'
      INCLUDE   'comiol.inc'
      INCLUDE   'commsk.inc'
      INCLUDE   'dum.inc'
      INCLUDE   'legau.inc'
      INCLUDE   'outcon.inc'
      INCLUDE   'spectr.inc'
C
      REAL*8       GPV(MGPP,NHEM,JG,NPH,2)
      EQUIVALENCE (GPV(1,1,1,1,1),Z(1))
      REAL*8       GV(MGP,JGGP,2),GX(MGP,JGGP),GY(MGP,JGGP)
      EQUIVALENCE (GV,GVE(1,1)),(GX,GVE(1,3)),(GY,GVE(1,3))
      REAL*8       DFAC(NPH),PHMN(NPH,2)
      REAL*8       CL(NCNT,2),GNORM(2),GINC(2),GMIN(2),GMAX(2)
      INTEGER      ICNT(2),ICSTYL(2)
      LOGICAL      LPRINT,LPLOT,LDUMP,LVEC
      LOGICAL      LBUG1,LBUG2,LBUG3
      CHARACTER    NAMES(NPH)*50,NAMC*50,TIMANN(2)*6
      INTEGER      IIDSIZE
      PARAMETER   (IIDSIZE=MGPP*NHEM*NPH*2+7)
      REAL*4       RIDATA(IIDSIZE)
      SAVE         NAMES,TIMANN
C
C----------------------------------------------------------------------
      INTEGER      I,N,J,JIN,IDPOS,K,IHEM,NT,IPTYP,NTMIN,IOUT
      INTEGER      JJ,JZ,IFAIL,IFRAME,IO,JOUNT
      REAL*8       TDAY,RKOUNT,RMTAPE,RNR,RMTAPE2
      REAL*8       SSUM,SDOT
C----------------------------------------------------------------------
      DATA (NAMES(N),N=1,NPH)/
     1 'SURFACE STRESS                           N/M2     '
     2,'SURFACE HEAT FLUX                        W/M2     '
     3,'SURFACE LATENT FLUX                      W/M2     '
     4,'CONVECTIVE RAINFALL                      MM/DAY   '
     5,'LARGE SCALE RAINFALL                     MM/DAY   '
     6,'SURFACE DOWNWARD SHORTWAVE FLUX          W/M2     '
     7,'SURFACE ENERGY BALANCE +VE DN            W/M2     '
     8,'SURFACE DOWNWARD LONGWAVE FLUX           W/M2     '
     9,'SURFACE UPWARD LONGWAVE FLUX             W/M2     '
     $,'TOP NET DOWNWARD SHORTWAVE FLUX          W/M2     '
     1,'TOP UPWARD LONGWAVE FLUX                 W/M2     '
     2,'SURFACE ALBEDO                           %        '
     3,'SURFACE TEMPERATURE                      K        '
     4,'DEEP SOIL TEMPERATURE                    K        '
     5,'SURFACE SPECIFIC HUMIDITY                KG/KG    '
     6,'SOIL MOISTURE                            M        '
     7,'SNOW DEPTH                               M (R=0.3)'
     8,'LOW CLOUD AMOUNT                         %        '
     9,'MID LEVEL CLOUD AMOUNT                   %        '
     $,'HIGH CLOUD AMOUNT                        %        '
     1,'CONVECTIVE CLOUD AMOUNT                  %        '
     6,'TOTAL RAINFALL                           MM/DAY   '
     :/
      DATA TIMANN/'(INST)','(AVER)'/
C
 6000 FORMAT(' PROCESSING SURFACE FIELDS  AT DAY  DOY'
     :     ,F10.2,F10.2,' KOUNT=',I10)
 6010 FORMAT(/' DAY DOY',F10.2,F10.2,
     :  '  NUMBER OF TIMESTEPS COMPLETED =',I10)
 6900 FORMAT(' ***ABORT IN PHYSOP AT DAY',F10.2
     :      ,' : LATITUDE COUNTER IN DATA =',I5,'  NOT',I5)
C
C     READ JG RECORDS OF 2D SURFACE FIELDS IF OUTPUT REQUIRED AT THIS
C     ANALYSIS TIME, OR SKIP IF NOT.
C     DAILY-AVERAGED FIELDS PRECEDE INSTANTANEOUS ON HISTORY FILE.
C
      CALL RCHECK(NCHIST,RNTAPE,KOUNT,TSPD)
      IF ((LPC7.AND.LOUT).OR.LPC9) THEN
         DO 10 J=1,JG
      IF (LSHIST) THEN
         READ (NCHIST) (RIDATA(JIN),JIN=1,6+MGPP*NHEM*NLR)
         RKOUNT=RIDATA(1)
         RMTAPE=RIDATA(2)
         DAY=RIDATA(3)
         DOY=RIDATA(4)
         RNR=RIDATA(5)
         IDPOS=6
         DO K=1,NLR
            DO IHEM=1,NHEM
               DO I=1,MGPP
                  GPV(I,IHEM,J,K,2)=RIDATA(IDPOS)
                  IDPOS=IDPOS+1
               ENDDO
            ENDDO
         ENDDO
         RMTAPE2=RIDATA(IDPOS)
         WRITE (*,*) 'MINIGRID: ',RKOUNT,RMTAPE,RMTAPE2
      ELSE
         READ (NCHIST) (RIDATA(JIN),JIN=1,6+MGPP*NHEM*NLR*2)
         RKOUNT=RIDATA(1)
         RMTAPE=RIDATA(2)
         DAY=RIDATA(3)
         DOY=RIDATA(4)
         RNR=RIDATA(5)
         IDPOS=6
         DO NT=2,1,-1
            DO K=1,NLR
               DO IHEM=1,NHEM
                  DO I=1,MGPP
                     GPV(I,IHEM,J,K,NT)=RIDATA(IDPOS)
                     IDPOS=IDPOS+1
                  ENDDO
               ENDDO
            ENDDO
         ENDDO
         RMTAPE2=RIDATA(IDPOS)
         WRITE (*,*) 'FULLGRID: ',RKOUNT,RMTAPE,RMTAPE2
      ENDIF
         IF (NINT(RNR).NE.J) THEN
            WRITE(6,6900) DAY,NINT(RNR),J
            CALL ABORT
         ENDIF
   10    CONTINUE
      ELSE
         DO 20 J=1,JG
   20    READ(NCHIST)
         RETURN
      ENDIF
C
C     WRITE HEADER AND DEFINE DIMENSIONALISING FACTORS AND CONSTANTS.
C
      IF (NPC7.NE.6) THEN
         IF (LOUTP) WRITE(NPC7,'(///)')
           WRITE(NPC7,6000) DAY,DOY,KOUNT
      ENDIF
      WRITE(6,6000) DAY,DOY,KOUNT
C
C        DO J=JG,1,-1
C        DO K=1,5
C        DO I=1,MGPP
C        IF (GPV(I,1,J,K,1).GT.1.0) PRINT *, 'HI',GPV(I,1,J,K,1)
C          PRINT *,'H',J,(REAL(GPV(I,1,J,4,2)),I=1,MGPP)
C        ENDDO
C        ENDDO
C         ENDDO
         DFAC(1)=P0
         DFAC(2)=CV*P0
         DFAC(3)=CV*P0
         DFAC(4)=TSPD
         DFAC(5)=TSPD
         DFAC(6)=1.0
      DO K=7,NPH
         DFAC(K)=1.
      ENDDO
      DFAC(13)=CT
      DFAC(14)=CT
      DFAC(16)=RADEA
      DFAC(17)=RADEA
      DFAC(2)=CV*P0
      DFAC(3)=CV*P0
         DFAC(22)=TSPD
C
      IPTYP=IZGTYP
      LVEC=.FALSE.
      IMSK=0
C
C     PROCESS DATA IN MAIN LOOPS OVER FIELDS.
C     NT=1,2 FOR INSTANTANEOUS THEN TIME-AVERAGED FIELDS.
C     IOUT=1,NPH FOR INDIVIDUAL FIELDS, WITH SPECIAL TREATMENT FOR
C     RAINFALL FIELDS.
C     ALSO CALCULATE GLOBAL MEANS IN PHMN FOR ROUTINE PHYSAV.
C
      NTMIN=1
      IF (LSHIST) NTMIN=2
      DO 200 NT=NTMIN,2
       IF (LPC7.AND.LOUTP) WRITE(NPC7,6010) DAY,DOY,KOUNT
C
      DO 50 IHEM=1,NHEM
      DO 50 J=1,JG
      DO 30 IOUT=1,NLR
      DO 30 I=1,MG
   30 GPV(I,IHEM,J,IOUT,NT)=GPV(I,IHEM,J,IOUT,NT)*DFAC(IOUT)
      DO 40 I=1,MGPP
 40   GPV(I,IHEM,J,NPH,NT)=GPV(I,IHEM,J,NCRR,NT)+GPV(I,IHEM,J,NLRR,NT)
   50 CONTINUE
C
      DO 100 IOUT=1,NPH
      LPRINT=LOUTP.AND.LPHGR(IOUT)
      LPLOT =LOUTF.AND.LPHPL(IOUT)
      LDUMP =LOUTA.AND.LPHPL(IOUT)
      IF (.NOT.(LPRINT.OR.LPLOT.OR.LDUMP.OR.LPC9)) GOTO 100
      N=1
      IFRAME=1
      ICSTYL(N)=0
      NAMES(IOUT)(35:40)=TIMANN(NT)
C
      DO 70 IHEM=1,NHEM
      DO 70 J=1,JG
      JJ=(2-IHEM)*J + (IHEM-1)*(JGGP+1-J)
      JZ=JJ-(IHEM-1)
      GVZ(JZ,1)=SSUM(MG,GPV(1,IHEM,J,IOUT,NT),1)*RMG
      DO 60 I=1,MG
   60 GV(I,JJ,N)=GPV(I,IHEM,J,IOUT,NT)
   70 CONTINUE
      IF (LPC9) PHMN(IOUT,NT)=SDOT(JGG,GVZ,1,GWT,1)
      IF (.NOT.(LPRINT.OR.LPLOT.OR.LDUMP)) GOTO 100
C
      CALL GVPREP(GV(1,1,N),PHFAC(IOUT),PHINC(IOUT),CL(1,N),NCNT
     :           ,GNORM(N),GINC(N),ICNT(N),GMIN(N),GMAX(N),IFAIL)
      IF (IFAIL.NE.0) CALL GFAIL(IFAIL,LPRINT,LPLOT,LDUMP
     :                          ,NAMES(IOUT)(1:30),ICNT(N))
C
      IF (LPRINT) THEN
         LBUG1=.FALSE.
         LBUG2=.FALSE.
         LBUG3=.TRUE.
         CALL GPRINT(GV(1,1,N),NAMES(IOUT),GMIN(N),GMAX(N)
     >        ,GNORM(N),NPC7)
         WRITE(NAMC,'(A,A,10X)') NAMES(IOUT)(1:30),NAMES(IOUT)(41:50)
         CALL XSECT(GVZ,1,NAMC,NPC7,PHFAC(IOUT),0.,LPRINT
     >        ,LBUG1,LBUG2,LBUG3,0,0,0)
      ENDIF
C
      IF (LDUMP) THEN
         IF (LNETCDF) THEN
            CALL WTFCDF(GV(1,1,N),MGP,JGGP,IPTYP,ICSTYL(N),0,5
     >           ,IFRAME,IMSK,GMIN(N),GMAX(N),GINC(N),DAY,DOY
     >           ,NAMES(IOUT),50,NPCA,IFAIL)
         ELSE
             CALL WTFUTF(GV(1,1,N),MGP,JGGP,IPTYP,ICSTYL(N),0,5
     >           ,IFRAME,IMSK,GMIN(N),GMAX(N),GINC(N),DAY,DOY
     >           ,NAMES(IOUT),50,NPCA,IFAIL)
         ENDIF
      ENDIF
C
      IF (.NOT.LPLOT) GOTO 100
      IF (IOUT.EQ.NCR.AND.LPLOT.AND.LPHPL(NLR)) GOTO 100
      IO=IOUT
      IF (IOUT.EQ.NLR) IO=NCR
C
  100 CONTINUE
  200 CONTINUE
C
C     WRITE GLOBAL MEANS TO CHANNEL NCGDG FOR OUTPUT BY ROUTINE PHYSAV
C     AT END OF RUN.
C
      IF (.NOT.LPC9) RETURN
      WRITE(NCGDG) KOUNT,DAY
      WRITE(NCGDG) PHMN
      IF (KOUNT.EQ.KEND) THEN
         JOUNT=-999
         TDAY=-999.
         WRITE(NCGDG) JOUNT,TDAY
      ENDIF
C
      RETURN
      END
