C----------------------------------------------------------------------
      SUBROUTINE XSECT(F,NLOUT,NAME,NPC,AFAC,CINC,LPRINT,LPLOT,LDUMP
     >                ,LNPLT,ICSTYL,IZDIM,IOUT)
C----------------------------------------------------------------------
      IMPLICIT   NONE
      INTEGER    NLOUT,NPC,ICSTYL,IZDIM,IOUT
C
C     TO PRINT, PLOT AND DUMP A ZONAL CROSS-SECTION.
C     USED FOR ALL ZONALLY AVERAGED FIELDS.  NO FILM OR DUMP TO UTF
C     (IE PRINT ONLY) IF NLOUT NOT EQUAL NL.
C     INCLUDES OROGRAPHIC MASK FIELD IF REQUIRED.
C     PRINTS HEMISPHERES SEPARATELY, BOTH RIGHT HANDED.
C     CERTAIN VALUES OF IOUT HAVE SPECIAL FUNCTIONS:
C        IOUT=2 : TEMPORARILY SUBTRACT 300K FROM POTENTIAL TEMP.
C                 ALSO SUPRESSES >NCNT CONTOUR WARNING.
C        IOUT=3 : TEMPORARILY SUBTRACT 273.15K FROM TEMPERATURE.
C        IOUT=18: ONLY 11 CONTOURS FOR TOTAL PERMITTED WAVENUMBER.
C                 ALSO SUPRESSES >NCNT CONTOUR WARNING.
C     TO AVOID ALL OF THESE FOR ADDED FIELDS USE E.G. IOUT=0.
C
      INCLUDE   'param1.inc'
      INCLUDE   'param2.inc'
      INTEGER    NCNT
      PARAMETER (NCNT=50)
      INCLUDE   'bats.inc'
      INCLUDE   'commsk.inc'
      INCLUDE   'outcon.inc'
      INCLUDE   'blank.inc'
C
      integer   i
      REAL*8    F(JGG,NLOUT),CL(NCNT)
      LOGICAL   LPRINT,LPLOT,LDUMP,LNPLT
      CHARACTER NAME*50,HEMANN(2)*30,DIMANN(4)*5
      SAVE      HEMANN,DIMANN
C----------------------------------------------------------------------
      INTEGER   J,L,MCNT,ICNT,IFAIL,IHEM,NPOLE,ILVTYP,IFRAME,IPTYP
      REAL*8    XOF,AFAC,CINC,GNORM,GINC,GMIN,GMAX,ZUNIT
C----------------------------------------------------------------------
C
      DATA HEMANN/'EQUATOR.............NORTH POLE'
     >           ,'SOUTH POLE.............EQUATOR'/
      DATA DIMANN/'     ','SIGMA','PRESS','THETA'/
C
 6000 FORMAT(/1X,A,1PE10.3,A)
 6001 FORMAT(/1X,A,F10.1,A)
 6002 FORMAT(/1X,A,F10.3,A)
 6010 FORMAT(6X,'MIN  MAX  =',1P,2(2X,E10.3),A)
 6020 FORMAT(' ZONAL MEAN',9X,15X,A)
 6030 FORMAT(' ZONAL MEAN ON ',A,15X,A)
 6040 FORMAT(1X,32I4)
C
C     SET SWITCHES.
C
      IPTYP=11
      IMSK=0
      IF (LMSK3.AND.(LPLOT.OR.LDUMP).AND.LNPLT.AND.(IZDIM.EQ.2)) IMSK=1
C
C     TEMPORARILY REMOVE OFFSET FOR TEMPERATURE FIELDS.
C
      IF (IOUT.EQ.2.OR.IOUT.EQ.3) THEN
         IF (IOUT.EQ.2) XOF=300.
         IF (IOUT.EQ.3) XOF=273.15
         DO 10 L=1,NLOUT
         DO 10 J=1,JGG
 10      F(J,L)=F(J,L)-XOF
      ENDIF
C
C     SET UP PRINTING FACTOR, CONTOUR LEVELS ETC.
C
      MCNT=NCNT
      IF (IOUT.EQ.18) MCNT=11
      CALL GPREP(F,JGG*NLOUT,AFAC,CINC,CL,MCNT,GNORM,GINC,ICNT
     >          ,GMIN,GMAX,IFAIL)
      IF (.NOT.((IOUT.EQ.2.OR.IOUT.EQ.18).AND.IFAIL.EQ.2)) THEN
         IF (IFAIL.NE.0) CALL GFAIL(IFAIL,LPRINT,LPLOT,LDUMP
     >                             ,NAME(1:30),ICNT)
      ENDIF
C
C     PRINT FIELD.
C
      IF (LPRINT) THEN
         ZUNIT=1./GNORM
         IF (ZUNIT.GT.1000..OR.ZUNIT.LE..099) THEN
            WRITE(NPC,6000) NAME(1:30),ZUNIT,NAME(31:45)
         ELSEIF (ZUNIT.GE.10.) THEN
            WRITE(NPC,6001) NAME(1:30),ZUNIT,NAME(31:45)
         ELSE
            WRITE(NPC,6002) NAME(1:30),ZUNIT,NAME(31:45)
         ENDIF
         WRITE(NPC,6010) GMIN,GMAX,NAME(31:45)
         DO 40 IHEM=1,NHEM
         IF (IZDIM.EQ.0) THEN
            WRITE(NPC,6020) HEMANN(IHEM)
         ELSE
            WRITE(NPC,6030) DIMANN(IZDIM+1),HEMANN(IHEM)
         ENDIF
         NPOLE=1+(NLAT-1)*INLAT
         IF (IHEM.EQ.1) THEN
            DO 20 L=1,NLOUT
 20         WRITE(NPC,6040) (NINT(F(J,L)*GNORM),J=NPOLE,1,-INLAT)
         ELSE
            DO 30 L=1,NLOUT
 30         WRITE(NPC,6040) (NINT(F(J,L)*GNORM),J=JGG,JGP,-INLAT)
         ENDIF
 40      CONTINUE
      ENDIF
C
C     ONLY PLOT OR DUMP FIELD IF DATA IS ON NL MODEL OR ISOBARIC LEVELS.
C
      IF ((NLOUT.NE.NL).AND.(NLOUT.NE.1)) RETURN
C
C     DUMP FIELD TO DATA TRANSFER FILE.
C
      IF (LDUMP) THEN
         ILVTYP=2
         IF (IZDIM.EQ.1) ILVTYP=0
         IFRAME=0
         IF (LNPLT) IFRAME=1
         IF (LNETCDF) THEN
            CALL WTFCDF(F,JGG,NLOUT,IPTYP,ICSTYL,0,ILVTYP,IFRAME,IMSK
     >           ,GMIN,GMAX,GINC,DAY,DOY,NAME,45,NPCA,IFAIL)
         ELSE
            CALL WTFUTF(F,JGG,NLOUT,IPTYP,ICSTYL,0,ILVTYP,IFRAME,IMSK
     >           ,GMIN,GMAX,GINC,DAY,DOY,NAME,45,NPCA,IFAIL)
         ENDIF
      ENDIF
C
C     ADD BACK THE TEMPORARILY REMOVED OFFSET.
C
      IF (IOUT.EQ.2.OR.IOUT.EQ.3) THEN
         DO 50 L=1,NLOUT
         DO 50 J=1,JGG
 50      F(J,L)=F(J,L)+XOF
      ENDIF
C
      RETURN
      END
