!=====================================================
! Calculate sin(latitude) and weight value
!=====================================================
      SUBROUTINE GWTLT(SIT,WEIGHT,J,JG,ISTAT)
!=====================================================
! Arguments
!----------------------------------------------------- 
      IMPLICIT NONE
! sin(latitude)
      REAL, INTENT(OUT) :: SIT
! weight value
      REAL, INTENT(OUT) :: WEIGHT
! index for latitude
      INTEGER, INTENT(IN) :: J
! number of latitude in one hemisphere
      INTEGER, INTENT(IN) :: JG
! Indicate if required convergence was achieved
      INTEGER, INTENT(OUT) :: ISTAT
!----------------------------------------------------- 
! Constants
!----------------------------------------------------- 
!      REAL(KIND=8), PARAMETER :: PI=3.14159265359
!      REAL(KIND=8), PARAMETER :: ACC=1.0D-23 
      REAL, PARAMETER :: PI=3.14159265359
      REAL, PARAMETER :: ACC=1.0D-16
      INTEGER, PARAMETER :: KMAX=50
!----------------------------------------------------- 
! Local variables
!----------------------------------------------------- 
!      REAL(KIND=8) :: SA
!      REAL(KIND=8) :: SB
!      REAL(KIND=8) :: SC
      REAL :: SA
      REAL :: SB
      REAL :: SC
      INTEGER :: GG
      INTEGER :: HH
!      REAL(KIND=8) :: X,AMM,AMN,
      REAL :: X,AMM,AMN,
     1 EM,RE,A,DD,DLT,DTH
!      REAL(KIND=8) :: TH
      REAL :: TH
      INTEGER :: K
      INTEGER :: ANN
      INTEGER :: N
!----------------------------------------------------- 
! Set some constants
!-----------------------------------------------------
      SA=DSQRT(.5)
      SB=DSQRT(1.5)
      SC=DSQRT(1.0/3.0)
      GG=4*JG+1
      HH=32*JG*JG
!----------------------------------------------------- 
! Continue the iterative process for 50 steps, unless sufficient
! convergence is reached before then. 
!-----------------------------------------------------
      ISTAT=0
      TH=PI*(2.*J-.5)/(1.0*GG)
      DTH=TH+(COS(TH)/SIN(TH))/(1.0*HH)
      X=DCOS(DTH)

      DO K=1,KMAX
         AMM=SA
         AMN=X*SB
         EM=SC
         ANN=1
         DO N=2,2*JG
            ANN=ANN+1
            RE=DSQRT(4.0*ANN*ANN-1.0)/ANN
            A=RE*(X*AMN-EM*AMM)
            AMM=AMN
            AMN=A
            EM=1.0/RE
         ENDDO
         DD=1.0*GG*EM*AMM-X*ANN*A
         DLT=(1.0-X*X)*A/DD
         IF (DABS(DLT).LT.ACC) GOTO 52
         X=X-DLT
      ENDDO
!----------------------------------------------------- 
! Reaching here indicates that required amount of convergence has
! not be met.
!-----------------------------------------------------
      ISTAT=1
52    CONTINUE
!----------------------------------------------------- 
! Set the values for output.
!-----------------------------------------------------
      WEIGHT=1.0*GG*(1.0-X*X)/(DD*DD)
      SIT=X
      RETURN
      END
